#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.


import os
import re

from job_common import Status
from job_common import StatisticsSnippet
from autosubmit.config.basicConfig import BasicConfig
from autosubmit.date.chunk_date_lib import *


class Job:
    """Class to handle all the tasks with Jobs at HPC.
       A job is created by default with a name, a jobid, a status and a type.
       It can have children and parents. The inheritance reflects the dependency between jobs.
       If Job2 must wait until Job1 is completed then Job2 is a child of Job1. Inversely Job1 is a parent of Job2 """

    def __init__(self, name, jobid, status, priority):
        self._queue = None
        self.queue_name = None
        self.section = None
        self.wallclock = None
        self.tasks = None
        self.threads = None
        self.processors = None
        self.chunk = None
        self.member = None
        self.date = None
        self.name = name
        self._long_name = None
        self.long_name = name
        self._short_name = None
        self.short_name = name

        self.id = jobid
        self.file = None
        self.status = status
        self.priority = priority
        self._parents = set()
        self._children = set()
        self.fail_count = 0
        self.expid = name.split('_')[0]
        self._complete = True
        self.parameters = dict()
        self._tmp_path = BasicConfig.LOCAL_ROOT_DIR + "/" + self.expid + "/" + BasicConfig.LOCAL_TMP_DIR + "/"
        self._ancestors = None

    def delete(self):
        del self.name
        del self._long_name
        del self._short_name
        del self.id
        del self.status
        del self.priority
        del self._parents
        del self._children
        del self.fail_count
        del self.expid
        del self._complete
        del self.parameters
        del self._tmp_path
        del self

    def print_job(self):
        Log.debug('NAME: %s' % self.name)
        Log.debug('JOBID: %s' % self.id)
        Log.debug('STATUS: %s' % self.status)
        Log.debug('TYPE: %s' % self.priority)
        Log.debug('PARENTS: %s' % [p.name for p in self.parents])
        Log.debug('CHILDREN: %s' % [c.name for c in self.children])
        Log.debug('FAIL_COUNT: %s' % self.fail_count)
        Log.debug('EXPID: %s' % self.expid)

    # Properties
    @property
    def parents(self):
        return self._parents

    def get_queue(self):
        if self.processors > 1:
            return self._queue
        else:
            return self._queue.get_serial_queue()

    def set_queue(self, value):
        self._queue = value

    @property
    def ancestors(self):
        if self._ancestors is None:
            self._ancestors = set()
            if self.has_parents():
                for parent in self.parents:
                    self._ancestors.add(parent)
                    for ancestor in parent.ancestors:
                        self._ancestors.add(ancestor)
        return self._ancestors

    @property
    def children(self):
        return self._children

    @property
    def long_name(self):
        """Returns the job long name"""
        # name is returned instead of long_name. Just to ensure backwards compatibility with experiments
        # that does not have long_name attribute.
        if hasattr(self, '_long_name'):
            return self._long_name
        else:
            return self.name

    @long_name.setter
    def long_name(self, value):
        self._long_name = value

    @property
    def short_name(self):
        """Returns the job short name"""
        return self._short_name

    @short_name.setter
    def short_name(self, value):
        n = value.split('_')
        if len(n) == 5:
            self._short_name = n[1][:6] + "_" + n[2][2:] + "_" + n[3] + n[4][:1]
        elif len(n) == 4:
            self._short_name = n[1][:6] + "_" + n[2][2:] + "_" + n[3][:1]
        elif len(n) == 2:
            self._short_name = n[1]
        else:
            self._short_name = n[0][:15]

    def log_job(self):
        Log.info("%s\t%s\t%s" % ("Job Name", "Job Id", "Job Status"))
        Log.info("%s\t\t%s\t%s" % (self.name, self.id, self.status))

    def get_all_children(self):
        """Returns a list with job's childrens and all it's descendents"""
        job_list = list()
        for job in self.children:
            job_list.append(job)
            job_list += job.get_all_children()
        # convert the list into a Set to remove duplicates and the again to a list
        return list(set(job_list))

    def print_parameters(self):
        Log.info(self.parameters)

    def inc_fail_count(self):
        self.fail_count += 1

    def add_parent(self, *new_parent):
        self._ancestors = None
        for parent in new_parent:
            self._parents.add(parent)
            parent.__add_child(self)

    def __add_child(self, new_children):
        self.children.add(new_children)

    def delete_parent(self, parent):
        self._ancestors = None
        # careful, it is only possible to remove one parent at a time
        self.parents.remove(parent)

    def delete_child(self, child):
        # careful it is only possible to remove one child at a time
        self.children.remove(child)

    def has_children(self):
        return self.children.__len__()

    def has_parents(self):
        return self.parents.__len__()

    def compare_by_status(self, other):
        return cmp(self.status(), other.status)

    def compare_by_type(self, other):
        return cmp(self.priority(), other.type)

    def compare_by_id(self, other):
        return cmp(self.id(), other.id)

    def compare_by_name(self, other):
        return cmp(self.name, other.name)

    def _get_from_completed(self, index):
        logname = self._tmp_path + self.name + '_COMPLETED'
        if os.path.exists(logname):
            split_line = open(logname).readline().split()
            if len(split_line) >= index + 1:
                return split_line[index]
            else:
                return 0
        else:
            return 0

    def check_end_time(self):
        return self._get_from_completed(0)

    def check_queued_time(self):
        return self._get_from_completed(1)

    def check_run_time(self):
        return self._get_from_completed(2)

    def check_failed_times(self):
        return self._get_from_completed(3)

    def check_fail_queued_time(self):
        return self._get_from_completed(4)

    def check_fail_run_time(self):
        return self._get_from_completed(5)

    def check_completion(self):
        """ Check the presence of *COMPLETED file and touch a Checked or failed file """
        logname = self._tmp_path + self.name + '_COMPLETED'
        if os.path.exists(logname):
            self._complete = True
            os.system('touch ' + self._tmp_path + self.name + 'Checked')
            self.status = Status.COMPLETED
        else:
            os.system('touch ' + self._tmp_path + self.name + 'Failed')
            self.status = Status.FAILED

    def remove_dependencies(self):
        """If Complete remove the dependency """
        if self._complete:
            self.status = Status.COMPLETED
            # job_logger.info("Job is completed, we are now removing the dependency in"
            # " his %s child/children:" % self.has_children())
            for child in self.children:
                # job_logger.debug("number of Parents:",child.has_parents())
                if child.get_parents().__contains__(self):
                    child.delete_parent(self)
        else:
            # job_logger.info("The checking in check_completion tell us that job %s has failed" % self.name)
            self.status = Status.FAILED

    def update_parameters(self):
        parameters = self.parameters
        parameters['JOBNAME'] = self.name
        parameters['FAIL_COUNT'] = str(self.fail_count)

        parameters['SDATE'] = self.date
        parameters['MEMBER'] = self.member
        if self.chunk is None:
            parameters['CHUNK'] = '1'
        else:
            parameters['CHUNK'] = self.chunk
            chunk = self.chunk
            total_chunk = int(parameters['NUMCHUNKS'])
            chunk_length = int(parameters['CHUNKSIZE'])
            chunk_unit = parameters['CHUNKSIZEUNIT'].lower()
            cal = parameters['CALENDAR'].lower()
            chunk_start = chunk_start_date(self.date, chunk, chunk_length, chunk_unit, cal)
            chunk_end = chunk_end_date(chunk_start, chunk_length, chunk_unit, cal)
            run_days = running_days(chunk_start, chunk_end, cal)
            chunk_end_days = previous_days(self.date, chunk_end, cal)
            day_before = previous_day(self.date, cal)
            chunk_end_1 = previous_day(chunk_end, cal)
            parameters['DAY_BEFORE'] = day_before
            parameters['Chunk_START_DATE'] = chunk_start
            parameters['Chunk_END_DATE'] = chunk_end_1
            parameters['RUN_DAYS'] = str(run_days)
            parameters['Chunk_End_IN_DAYS'] = str(chunk_end_days)

            chunk_start_m = chunk_start_month(chunk_start)
            chunk_start_y = chunk_start_year(chunk_start)

            parameters['Chunk_START_YEAR'] = str(chunk_start_y)
            parameters['Chunk_START_MONTH'] = str(chunk_start_m)
            if total_chunk == chunk:
                parameters['Chunk_LAST'] = 'TRUE'
            else:
                parameters['Chunk_LAST'] = 'FALSE'

        parameters['NUMPROC'] = self.processors
        parameters['NUMTHREADS'] = self.threads
        parameters['NUMTASK'] = self.tasks
        parameters['WALLCLOCK'] = self.wallclock
        parameters['TASKTYPE'] = self.section

        queue = self.get_queue()
        parameters['HPCUSER'] = queue.user
        parameters['HPCPROJ'] = queue.project
        parameters['HPCTYPE'] = queue.type
        parameters['SCRATCH_DIR'] = queue.scratch

        self.parameters = parameters

        return parameters

    def update_content(self, project_dir):
        if self.parameters['PROJECT_TYPE'].lower() != "none":
            dir_templates = project_dir
            template = file(os.path.join(dir_templates, self.file), 'r').read()
        else:
            template = ''
        queue = self.get_queue()
        if queue.name == 'local':
            stats_header = StatisticsSnippet.AS_HEADER_LOC
            stats_tailer = StatisticsSnippet.AS_TAILER_LOC
        else:
            stats_header = StatisticsSnippet.AS_HEADER_REM
            stats_tailer = StatisticsSnippet.AS_TAILER_REM

        template_content = ''.join([queue.get_header(self),
                                   stats_header,
                                   template,
                                   stats_tailer])

        return template_content

    def create_script(self, as_conf):
        parameters = self.update_parameters()
        template_content = self.update_content(as_conf.get_project_dir())
        # print "jobType: %s" % self._type
        # print template_content

        for key, value in parameters.items():
            # print "%s:\t%s" % (key,parameters[key])
            template_content = template_content.replace("%" + key + "%", str(parameters[key]))

        scriptname = self.name + '.cmd'
        file(self._tmp_path + scriptname, 'w').write(template_content)

        return scriptname

    def check_script(self, as_conf):
        parameters = self.update_parameters()
        template_content = self.update_content(as_conf.get_project_dir())

        variables = re.findall('%' + '(\w+)' + '%', template_content)
        # variables += re.findall('%%'+'(.+?)'+'%%', template_content)
        out = set(parameters).issuperset(set(variables))

        if not out:
            Log.warning("The following set of variables to be substituted in template script is not part of "
                        "parameters set: ")
            Log.warning(set(variables) - set(parameters))
        else:
            self.create_script(as_conf)

        return out


if __name__ == "__main__":
    mainJob = Job('uno', '1', Status.READY, 0)
    job1 = Job('uno', '1', Status.READY, 0)
    job2 = Job('dos', '2', Status.READY, 0)
    job3 = Job('tres', '3', Status.READY, 0)
    jobs = [job1, job2, job3]
    mainJob.add_parent(*jobs)
    Log.info(mainJob.parents)
    # mainJob.set_children(jobs)
    # job1.__add_child(mainJob)
    # job2.__add_child(mainJob)
    # job3.__add_child(mainJob)
    Log.info(mainJob.get_all_children())
    Log.info(mainJob.children)
    job3.check_completion()
    Log.info("Number of Parents: ", mainJob.has_parents())
    Log.info("number of children : ", mainJob.has_children())
    mainJob.print_job()
    mainJob.delete()
#



#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
from ConfigParser import SafeConfigParser
import argparse
from commands import getstatusoutput
import json
import time
import cPickle
import signal
import os
import sys
import shutil
import re
import random
from pkg_resources import require, resource_listdir, resource_exists, resource_string
from time import strftime
from distutils.util import strtobool

from pyparsing import nestedExpr

from autosubmit.config.basicConfig import BasicConfig
from autosubmit.config.config_common import AutosubmitConfig
from autosubmit.job.job_common import Status
from autosubmit.git.git_common import AutosubmitGit
from autosubmit.job.job_list import JobList
from autosubmit.config.log import Log
from autosubmit.database.db_common import create_db
from autosubmit.database.db_common import new_experiment
from autosubmit.database.db_common import copy_experiment
from autosubmit.database.db_common import delete_experiment
from autosubmit.monitor.monitor import Monitor
from autosubmit.config.config_parser import expdef_parser


class Autosubmit:
    # Get the version number from the relevant file. If not, from autosubmit package
    scriptdir = os.path.abspath(os.path.dirname(sys.argv[0]))
    version_path = os.path.join(scriptdir, '..', 'VERSION')
    if os.path.isfile(version_path):
        with open(version_path) as f:
            autosubmit_version = f.read().strip()
    else:
        autosubmit_version = require("autosubmit")[0].version

    @staticmethod
    def parse_args():
        BasicConfig.read()

        parser = argparse.ArgumentParser(description='Main executable for autosubmit. ')
        parser.add_argument('-v', '--version', action='version', version=Autosubmit.autosubmit_version,
                            help="return Autosubmit's version number and exit")
        parser.add_argument('-lf', '--logfile', choices=('EVERYTHING', 'DEBUG', 'INFO', 'RESULT', 'USER_WARNING',
                                                         'WARNING', 'ERROR', 'CRITICAL', 'NO_LOG'),
                            default='INFO', type=str,
                            help="set log to file level.")
        parser.add_argument('-lc', '--logconsole', choices=('EVERYTHING', 'DEBUG', 'INFO', 'RESULT', 'USER_WARNING',
                                                            'WARNING', 'ERROR', 'CRITICAL', 'NO_LOG'),
                            default='INFO', type=str,
                            help="set log file level")

        subparsers = parser.add_subparsers(dest='command')

        # Run
        subparser = subparsers.add_parser('run', description="run specified experiment")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')

        # Expid
        subparser = subparsers.add_parser('expid', description="Creates a new experiment")
        group = subparser.add_mutually_exclusive_group()
        group.add_argument('-y', '--copy', help='makes a copy of the specified experiment')
        group.add_argument('-dm', '--dummy', action='store_true',
                           help='creates a new experiment with default values, usually for testing')

        subparser.add_argument('-H', '--HPC', required=True,
                               help='Specifies the HPC to use for the experiment')
        subparser.add_argument('-d', '--description', type=str, required=True,
                               help='A description for the experiment to store in the database.')

        # Delete
        subparser = subparsers.add_parser('delete', description="delete specified experiment")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')
        subparser.add_argument('-f', '--force', action='store_true', help='delete experiment without confirmation')

        # Monitor
        subparser = subparsers.add_parser('monitor', description="plots specified experiment")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')
        subparser.add_argument('-j', '--joblist', required=True, help='joblist to print')
        subparser.add_argument('-o', '--output', required=True, choices=('pdf', 'png', 'ps'), default='pdf',
                               help='type of output for generated plot')

        # Stats
        subparser = subparsers.add_parser('stats', description="plots statistics for specified experiment")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')
        subparser.add_argument('-j', '--joblist', required=True, help='joblist to print')
        subparser.add_argument('-o', '--output', required=True, choices=('pdf', 'png', 'ps'), default='pdf',
                               help='type of output for generated plot')

        # Clean
        subparser = subparsers.add_parser('clean', description="clean specified experiment")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')
        subparser.add_argument('-pr', '--project', action="store_true", help='clean project')
        subparser.add_argument('-p', '--plot', action="store_true",
                               help='clean plot, only 2 last will remain')
        subparser.add_argument('-s', '--stats', action="store_true",
                               help='clean stats, only last will remain')

        # Recovery
        subparser = subparsers.add_parser('recovery', description="recover specified experiment")
        subparser.add_argument('-e', '--expid', type=str, required=True, help='experiment identifier')
        subparser.add_argument('-j', '--joblist', type=str, required=True, help='Job list')
        subparser.add_argument('-g', '--get', action="store_true", default=False,
                               help='Get completed files to synchronize pkl')
        subparser.add_argument('-s', '--save', action="store_true", default=False, help='Save changes to disk')

        # Check
        subparser = subparsers.add_parser('check', description="check configuration for specified experiment")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')

        # Create
        subparser = subparsers.add_parser('create', description="create specified experiment joblist")
        subparser.add_argument('-e', '--expid', required=True, help='experiment identifier')
        subparser.add_argument('-np', '--noplot', action='store_true', default=False, help='omit plot')

        # Configure
        subparser = subparsers.add_parser('configure', description="configure database and path for autosubmit. It "
                                                                   "can be done at machine, user or local level (by "
                                                                   "default at machine level)")
        subparser.add_argument('-db', '--databasepath', default=None, help='path to database. If not supplied, '
                                                                           'it will be prompt for it')
        subparser.add_argument('-lr', '--localrootpath', default=None, help='path to store experiments. If not '
                                                                            'supplied, it will be prompt for it')
        group = subparser.add_mutually_exclusive_group()
        group.add_argument('-u', '--user', action="store_true", help='configure only for this user')
        group.add_argument('-l', '--local', action="store_true", help='configure only for using Autosubmit from this '
                                                                      'path')

        # Install
        subparsers.add_parser('install', description='install database and scripts needed for autosubmit')

        # Change_pkl
        subparser = subparsers.add_parser('change_pkl', description="change job status for an experiment")
        subparser.add_argument('-e', '--expid', type=str, required=True, help='experiment identifier')
        subparser.add_argument('-j', '--joblist', type=str, required=True, help='Job list')
        subparser.add_argument('-s', '--save', action="store_true", default=False, help='Save changes to disk')
        subparser.add_argument('-t', '--status_final',
                               choices=('READY', 'COMPLETED', 'WAITING', 'SUSPENDED', 'FAILED', 'UNKNOWN'),
                               required=True,
                               help='Supply the target status')
        group1 = subparser.add_mutually_exclusive_group(required=True)
        group1.add_argument('-l', '--list', type=str,
                            help='Alternative 1: Supply the list of job names to be changed. Default = "Any". '
                                 'LIST = "b037_20101101_fc3_21_sim b037_20111101_fc4_26_sim"')
        group1.add_argument('-f', '--filter', action="store_true",
                            help='Alternative 2: Supply a filter for the job list. See help of filter arguments: '
                                 'chunk filter, status filter or type filter')
        group2 = subparser.add_mutually_exclusive_group(required=False)
        group2.add_argument('-fc', '--filter_chunks', type=str,
                            help='Supply the list of chunks to change the status. Default = "Any". '
                                 'LIST = "[ 19601101 [ fc0 [1 2 3 4] fc1 [1] ] 19651101 [ fc0 [16-30] ] ]"')
        group2.add_argument('-fs', '--filter_status', type=str,
                            choices=('Any', 'READY', 'COMPLETED', 'WAITING', 'SUSPENDED', 'FAILED', 'UNKNOWN'),
                            help='Select the original status to filter the list of jobs')
        group2.add_argument('-ft', '--filter_type', type=str,
                            help='Select the job type to filter the list of jobs')

        # Test
        subparser = subparsers.add_parser('test', description='test experiment')
        subparser.add_argument('-e', '--expid', type=str, required=True, help='experiment identifier')
        subparser.add_argument('-c', '--chunks', required=True, help='chunks to run')
        subparser.add_argument('-m', '--member', help='member to run')
        subparser.add_argument('-s', '--stardate', help='stardate to run')
        subparser.add_argument('-H', '--HPC', help='HPC to run experiment on it')
        subparser.add_argument('-b', '--branch', help='branch of git to run (or revision from subversion)')

        args = parser.parse_args()

        Log.set_console_level(args.logconsole)
        Log.set_file_level(args.logfile)

        if args.command == 'run':
            Autosubmit.run_experiment(args.expid)
        elif args.command == 'expid':
            Autosubmit.expid(args.HPC, args.description, args.copy, args.dummy)
        elif args.command == 'delete':
            Autosubmit.delete(args.expid, args.force)
        elif args.command == 'monitor':
            Autosubmit.monitor(args.expid, args.joblist, args.output)
        elif args.command == 'stats':
            Autosubmit.statistics(args.expid, args.joblist, args.output)
        elif args.command == 'clean':
            Autosubmit.clean(args.expid, args.project, args.plot, args.stats)
        elif args.command == 'recovery':
            Autosubmit.recovery(args.expid, args.joblist, args.save, args.get)
        elif args.command == 'check':
            Autosubmit.check(args.expid)
        elif args.command == 'create':
            Autosubmit.create(args.expid, args.noplot)
        elif args.command == 'configure':
            Autosubmit.configure(args.databasepath, args.localrootpath, args.user, args.local)
        elif args.command == 'install':
            Autosubmit.install()
        elif args.command == 'change_pkl':
            Autosubmit.change_pkl(args.expid, args.joblist, args.save, args.status_final, args.list, args.filter,
                                  args.filter_chunks, args.filter_status, args.filter_section)
        elif args.command == 'test':
            Autosubmit.test(args.expid, args.chunks, args.member, args.stardate, args.HPC, args.branch)

    @staticmethod
    def delete_expid(expid_delete):
        Log.info("Removing experiment directory...")
        try:
            shutil.rmtree(BasicConfig.LOCAL_ROOT_DIR + "/" + expid_delete)
        except OSError:
            pass
        Log.info("Deleting experiment from database...")
        delete_experiment(expid_delete)
        Log.result("Experiment {0} deleted".format(expid_delete))

    @staticmethod
    def expid(hpc, description, copy, dummy):
        BasicConfig.read()

        log_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, 'expid{0}.log'.format(os.getuid()))
        try:
            Log.set_file(log_path)
        except IOError as e:
            Log.error("Can not create log file in path {0}: {1}".format(log_path, e.message))
        exp_id = None
        if description is None:
            Log.error("Missing experiment description.")
            exit(2)
        if hpc is None:
            Log.error("Missing HPC.")
            exit(1)
        if not copy:
            exp_id = new_experiment(hpc, description)
            try:
                os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id)

                os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + '/conf')
                Log.info("Copying config files...")
                # autosubmit config and experiment copyed from AS.
                files = resource_listdir('autosubmit.config', 'files')
                for filename in files:
                    if resource_exists('autosubmit.config', 'files/' + filename):
                        index = filename.index('.')
                        new_filename = filename[:index] + "_" + exp_id + filename[index:]
                        content = resource_string('autosubmit.config', 'files/' + filename)
                        Log.debug(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/conf/" + new_filename)
                        file(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/conf/" + new_filename, 'w').write(content)
                Autosubmit._prepare_conf_files(exp_id, hpc, Autosubmit.autosubmit_version, dummy)
            except (OSError, IOError) as e:
                Log.error("Can not create experiment: {0}\nCleaning...".format(e.message))
                Autosubmit.delete_expid(exp_id)
                exit(1)
        else:
            try:
                if os.path.exists(BasicConfig.LOCAL_ROOT_DIR + "/" + copy):
                    exp_id = copy_experiment(copy, hpc, description)
                    os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id)
                    os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + '/conf')
                    Log.info("Copying previous experiment config directories")
                    files = os.listdir(BasicConfig.LOCAL_ROOT_DIR + "/" + copy + "/conf")
                    for filename in files:
                        if os.path.isfile(BasicConfig.LOCAL_ROOT_DIR + "/" + copy + "/conf/" + filename):
                            new_filename = filename.replace(copy, exp_id)
                            content = file(BasicConfig.LOCAL_ROOT_DIR + "/" + copy + "/conf/" + filename, 'r').read()
                            file(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/conf/" + new_filename,
                                 'w').write(content)
                    Autosubmit._prepare_conf_files(exp_id, hpc, Autosubmit.autosubmit_version, dummy)
                else:
                    Log.critical("The previous experiment directory does not exist")
                    sys.exit(1)
            except (OSError, IOError) as e:
                Log.error("Can not create experiment: {0}\nCleaning...".format(e))
                Autosubmit.delete_expid(exp_id)
                exit(1)

        Log.debug("Creating temporal directory...")
        os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/" + "tmp")

        Log.debug("Creating pkl directory...")
        os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/" + "pkl")

        Log.debug("Creating plot directory...")
        os.mkdir(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/" + "plot")
        os.chmod(BasicConfig.LOCAL_ROOT_DIR + "/" + exp_id + "/" + "plot", 0o775)
        Log.result("Experiment registered successfully")
        Log.user_warning("Remember to MODIFY the config files!")
        return exp_id

    @staticmethod
    def delete(expid, force):
        if os.path.exists(BasicConfig.LOCAL_ROOT_DIR + "/" + expid):
            if force or Autosubmit._user_yes_no_query("Do you want to delete " + expid + " ?"):
                Autosubmit.delete_expid(expid)
            else:
                Log.info("Quitting...")
                sys.exit(1)
        else:
            Log.error("The experiment does not exist")
            sys.exit(1)

    @staticmethod
    def run_experiment(expid):
        """This is the main script of autosubmit. All the stream of execution is handled here
        (submitting all the jobs properly and repeating its execution in case of failure)."""
        if expid is None:
            Log.critical("Missing expid.")
        BasicConfig.read()
        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR,
                                  'autosubmit.log'))
        os.system('clear')

        as_conf = AutosubmitConfig(expid)
        as_conf.check_conf()

        project_type = as_conf.get_project_type()
        if project_type != "none":
            # Check proj configuration
            as_conf.check_proj()

        expid = as_conf.get_expid()
        hpcarch = as_conf.get_platform()
        max_jobs = as_conf.get_total_jobs()
        max_waiting_jobs = as_conf.get_max_waiting_jobs()
        safetysleeptime = as_conf.get_safetysleeptime()
        retrials = as_conf.get_retrials()
        rerun = as_conf.get_rerun()

        queues = AutosubmitConfig.read_queues_conf(expid)

        Log.debug("The Experiment name is: %s" % expid)
        Log.debug("Total jobs to submit: %s" % max_jobs)
        Log.debug("Maximum waiting jobs in queues: %s" % max_waiting_jobs)
        Log.debug("Sleep: %s" % safetysleeptime)
        Log.debug("Retrials: %s" % retrials)
        Log.info("Starting job submission...")

        for queue in queues:
            signal.signal(signal.SIGQUIT, queues[queue].smart_stop)
            signal.signal(signal.SIGINT, queues[queue].normal_stop)

        if rerun == 'false':
            filename = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + '/pkl/job_list_' + expid + '.pkl'
        else:
            filename = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + '/pkl/rerun_job_list_' + expid + '.pkl'
        Log.debug(filename)

        # the experiment should be loaded as well
        if os.path.exists(filename):
            joblist = cPickle.load(file(filename, 'rw'))
            Log.debug("Starting from joblist pickled in %s " % filename)
        else:
            Log.error("The pickle file %s necessary does not exist." % filename)
            sys.exit()

        Log.debug("Length of joblist: %s" % len(joblist))

        # Load parameters
        Log.debug("Loading parameters...")
        parameters = as_conf.load_parameters()
        Log.debug("Updating parameters...")
        joblist.update_parameters(parameters)
        # check the job list script creation
        Log.debug("Checking experiment templates...")

        for job in joblist.get_job_list():
            if job.queue_name is None:
                job.queue_name = hpcarch
            job.set_queue(queues[job.queue_name])

        if joblist.check_scripts(as_conf):
            Log.result("Experiment templates check PASSED!")
        else:
            Log.error("Experiment templates check FAILED!")
            sys.exit()

        # check the availability of the Queues
        for queue in queues:
            queues[queue].check_remote_log_dir()

        #########################
        # AUTOSUBMIT - MAIN LOOP
        #########################
        # Main loop. Finishing when all jobs have been submitted
        while joblist.get_active():
            active = len(joblist.get_running())
            waiting = len(joblist.get_submitted() + joblist.get_queuing())
            available = max_waiting_jobs - waiting

            # reload parameters changes
            Log.debug("Reloading parameters...")
            as_conf.reload()
            parameters = as_conf.load_parameters()
            joblist.update_parameters(parameters)

            # variables to be updated on the fly
            max_jobs = as_conf.get_total_jobs()
            Log.debug("Total jobs: {0}".format(max_jobs))
            total_jobs = len(joblist.get_job_list())
            Log.info("\n{0} of {1} jobs remaining ({2})".format(total_jobs-len(joblist.get_completed()), total_jobs,
                                                                strftime("%H:%M")))
            safetysleeptime = as_conf.get_safetysleeptime()
            Log.debug("Sleep: %s" % safetysleeptime)
            retrials = as_conf.get_retrials()
            Log.debug("Number of retrials: %s" % retrials)

            # read FAIL_RETRIAL number if, blank at creation time put a given number
            # check availability of machine, if not next iteration after sleep time
            # check availability of jobs, if no new jobs submited and no jobs available, then stop

            # ??? why
            joblist.save()

            Log.info("Active jobs in queues:\t%s" % active)
            Log.info("Waiting jobs in queues:\t%s" % waiting)

            if available == 0:
                Log.debug("There's no room for more jobs...")
            else:
                Log.debug("We can safely submit %s jobs..." % available)

            ######################################
            # AUTOSUBMIT - ALREADY SUBMITTED JOBS
            ######################################
            # get the list of jobs currently in the Queue
            jobinqueue = joblist.get_in_queue()
            Log.info("Number of jobs in queue: %s" % str(len(jobinqueue)))

            for job in jobinqueue:

                job.print_job()
                Log.debug("Number of jobs in queue: %s" % str(len(jobinqueue)))
                # Check queue availability
                job_queue = job.get_queue()
                queueavail = job_queue.check_host()
                if not queueavail:
                    Log.debug("There is no queue available")
                else:
                    status = job_queue.check_job(job.id)
                    if status == Status.COMPLETED:
                        Log.debug("This job seems to have completed...checking")
                        job_queue.get_completed_files(job.name)
                        job.check_completion()
                    else:
                        job.status = status
                    if job.status is Status.QUEUING:
                        Log.info("Job %s is QUEUING", job.name)
                    elif job.status is Status.RUNNING:
                        Log.info("Job %s is RUNNING", job.name)
                    elif job.status is Status.COMPLETED:
                        Log.result("Job %s is COMPLETED", job.name)
                    elif job.status is Status.FAILED:
                        Log.user_warning("Job %s is FAILED", job.name)
                        # Uri add check if status UNKNOWN and exit if you want
                        # after checking the jobs , no job should have the status "submitted"
                        # Uri throw an exception if this happens (warning type no exit)

            # explain it !!
            joblist.update_list()

            ##############################
            # AUTOSUBMIT - JOBS TO SUBMIT
            ##############################
            # get the list of jobs READY
            jobsavail = joblist.get_ready()

            if min(available, len(jobsavail)) == 0:
                Log.debug("There is no job READY or available")
                Log.debug("Number of jobs ready: %s" % len(jobsavail))
                Log.debug("Number of jobs available in queue: %s" % available)
            elif min(available, len(jobsavail)) > 0 and len(jobinqueue) <= max_jobs:
                Log.info("\nStarting to submit %s job(s)" % min(available, len(jobsavail)))
                # should sort the jobsavail by priority Clean->post->sim>ini
                # s = sorted(jobsavail, key=lambda k:k.name.split('_')[1][:6])
                # probably useless to sort by year before sorting by type
                s = sorted(jobsavail, key=lambda k: k.long_name.split('_')[1][:6])

                list_of_jobs_avail = sorted(s, key=lambda k: k.priority, reverse=True)

                for job in list_of_jobs_avail[0:min(available, len(jobsavail), max_jobs - len(jobinqueue))]:
                    Log.debug(job.name)
                    scriptname = job.create_script(as_conf)
                    Log.debug(scriptname)

                    job_queue = job.get_queue()
                    queueavail = job_queue.check_host()
                    if not queueavail:
                        Log.warning("Queue {0} is not available".format(job_queue.name))
                    else:
                        job_queue.send_script(scriptname)
                        job.id = job_queue.submit_job(scriptname)
                        # set status to "submitted"
                        job.status = Status.SUBMITTED
                        Log.info("%s submitted\n", job.name)

            time.sleep(safetysleeptime)

        Log.info("No more jobs to run.")
        if len(joblist.get_failed()) > 0:
            Log.info("Some jobs have failed and reached maximun retrials")
            return False
        else:
            Log.result("Run successful")
            return True

    @staticmethod
    def monitor(expid, root_name, output):
        BasicConfig.read()
        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR, 'monitor.log'))
        filename = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + '/pkl/' + root_name + '_' + expid + '.pkl'
        Log.info("Getting job list...")
        jobs = cPickle.load(file(filename, 'r'))
        if not isinstance(jobs, type([])):
            jobs = jobs.get_job_list()

        Log.info("Plotting...")
        monitor_exp = Monitor()
        monitor_exp.generate_output(expid, jobs, output)
        Log.result("Plot ready")

    @staticmethod
    def statistics(expid, root_name, output):
        BasicConfig.read()
        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR,
                                  'statistics.log'))
        Log.info("Loading jobs...")
        filename = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + '/pkl/' + root_name + '_' + expid + '.pkl'
        jobs = cPickle.load(file(filename, 'r'))
        # if not isinstance(jobs, type([])):
        #     jobs = [job for job in jobs.get_finished() if job.type == Type.SIMULATION]

        if len(jobs.get_job_list()) > 0:
            Log.info("Plotting stats...")
            monitor_exp = Monitor()
            monitor_exp.generate_output_stats(expid, jobs.get_job_list(), output)
            Log.result("Stats plot ready")
        else:
            Log.info("There are no COMPLETED jobs...")

    @staticmethod
    def clean(expid, project, plot, stats):
        BasicConfig.read()
        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR,
                                  'finalise_exp.log'))
        if project:
            autosubmit_config = AutosubmitConfig(expid)
            autosubmit_config.check_conf()
            project_type = autosubmit_config.get_project_type()
            if project_type == "git":
                autosubmit_config.check_proj()
                Log.info("Registering commit SHA...")
                autosubmit_config.set_git_project_commit()
                autosubmit_git = AutosubmitGit(expid[0])
                Log.info("Cleaning GIT directory...")
                autosubmit_git.clean_git()
            else:
                Log.info("No project to clean...\n")
        if plot:
            Log.info("Cleaning plots...")
            monitor_autosubmit = Monitor()
            monitor_autosubmit.clean_plot(expid)
        if stats:
            Log.info("Cleaning stats directory...")
            monitor_autosubmit = Monitor()
            monitor_autosubmit.clean_stats(expid)

    @staticmethod
    def recovery(expid, root_name, save, get):
        BasicConfig.read()

        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR,
                                  'recovery.log'))

        Log.info('Recovering experiment {0}'.format(expid))
        l1 = cPickle.load(file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, 'pkl', root_name + "_" + expid + ".pkl"),
                               'r'))

        as_conf = AutosubmitConfig(expid)
        as_conf.check_conf()

        hpcarch = as_conf.get_platform()

        if get:
            queues = AutosubmitConfig.read_queues_conf(expid)

            for job in l1.get_active():
                if job.queue_name is None:
                    job.queue_name = hpcarch
                job.set_queue(queues[job.queue_name])

                if job.get_queue().get_completed_files(job.name):
                    job.status = Status.COMPLETED
                    Log.info("CHANGED job '{0}' status to COMPLETED".format(job.name))
                elif job.status != Status.SUSPENDED:
                    job.status = Status.READY
                    job.set_fail_count(0)
                    Log.info("CHANGED job '{0}' status to READY".format(job.name))

            sys.setrecursionlimit(50000)
            l1.update_list()
            cPickle.dump(l1, file(BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/pkl/" + root_name + "_" + expid + ".pkl",
                                  'w'))

        if save:
            l1.update_from_file()
        else:
            l1.update_from_file(False)

        if save:
            sys.setrecursionlimit(50000)
            cPickle.dump(l1, file(BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/pkl/" + root_name + "_" + expid +
                                  ".pkl", 'w'))
        Log.result("Recovery finalized")
        monitor_exp = Monitor()
        monitor_exp.generate_output(expid, l1.get_job_list())

    @staticmethod
    def check(expid):
        BasicConfig.read()
        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR, 'check_exp.log'))
        as_conf = AutosubmitConfig(expid)
        as_conf.check_conf()
        project_type = as_conf.get_project_type()
        if project_type != "none":
            as_conf.check_proj()

        # print "Checking experiment configuration..."
        # if as_conf.check_parameters():
        #     print "Experiment configuration check PASSED!"
        # else:
        #     print "Experiment configuration check FAILED!"
        #     print "WARNING: running after FAILED experiment configuration check is at your own risk!!!"

        Log.info("Checking experiment templates...")
        if Autosubmit._check_templates(as_conf):
            Log.result("Experiment templates check PASSED!")
        else:
            Log.critical("Experiment templates check FAILED!")
            Log.warning("Running after FAILED experiment templates check is at your own risk!!!")

    @staticmethod
    def configure(database_path, local_root_path,  user, local):
        home_path = os.path.expanduser('~')
        while database_path is None:
            database_path = raw_input("Introduce Database path: ")
        database_path = database_path.replace('~', home_path)
        if not os.path.exists(database_path):
            Log.error("Database path does not exist.")
            exit(1)

        while local_root_path is None:
            local_root_path = raw_input("Introduce Local Root path: ")
        local_root_path = local_root_path.replace('~', home_path)
        if not os.path.exists(local_root_path):
            Log.error("Local Root path does not exist.")
            exit(1)

        if user:
            path = home_path
        elif local:
            path = '.'
        else:
            path = '/etc'
        path = os.path.join(path, '.autosubmitrc')

        config_file = open(path, 'w')
        Log.info("Writing configuration file...")
        try:
            parser = SafeConfigParser()
            parser.add_section('database')
            parser.set('database', 'path', database_path)
            parser.add_section('local')
            parser.set('local', 'path', local_root_path)
            parser.write(config_file)
            config_file.close()
            Log.result("Configuration file written successfully")
        except (IOError, OSError) as e:
            Log.critical("Can not write config file: {0}".format(e.message))

    @staticmethod
    def install():
        BasicConfig.read()
        if not os.path.exists(BasicConfig.DB_PATH):
            Log.info("Creating autosubmit database...")
            try:
                qry = resource_string('autosubmit.database', 'data/autosubmit.sql')
                create_db(qry)
                Log.result("Autosubmit database creatd successfully")
            except Exception as e:
                Log.critical("Can not write database file: {0}".format(e.message))
        else:
            Log.error("Database already exists.")
            exit(1)

    @staticmethod
    def create(expid, noplot):
        BasicConfig.read()
        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR,
                                  'create_exp.log'))
        as_conf = AutosubmitConfig(expid)
        as_conf.check_conf()

        expid = as_conf.get_expid()
        project_type = as_conf.get_project_type()

        if project_type == "git":
            git_project_origin = as_conf.get_git_project_origin()
            git_project_branch = as_conf.get_git_project_branch()
            project_path = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/" + BasicConfig.LOCAL_PROJ_DIR
            if os.path.exists(project_path):
                Log.debug("The project folder exists. SKIPPING...")
                Log.info("Using project folder: %s" % project_path)
            else:
                os.mkdir(project_path)
                Log.debug("The project folder %s has been created." % project_path)
                Log.info("Cloning %s into %s" % (git_project_branch + " " + git_project_origin, project_path))
                (status, output) = getstatusoutput("cd " + project_path + "; git clone -b " + git_project_branch +
                                                   " " + git_project_origin)
                if status:
                    os.rmdir(project_path)
                    Log.error("Can not clone %s into %s" % (git_project_branch + " " + git_project_origin,
                                                            project_path))
                    exit(1)

                Log.debug("%s" % output)
                git_project_name = output[output.find("'")+1:output.find("...")-1]
                (status, output) = getstatusoutput("cd " + project_path + "/" + git_project_name +
                                                   "; git submodule update --remote --init")
                if status:
                    os.rmdir(project_path)
                    Log.error("Can not clone %s into %s" % (git_project_branch + " " + git_project_origin,
                                                            project_path))
                    exit(1)
                Log.debug("%s" % output)

                (status, output) = getstatusoutput("cd " + project_path + "/" + git_project_name +
                                                   "; git submodule foreach -q 'branch=\"$(git config "
                                                   "-f $toplevel/.gitmodules submodule.$name.branch)\"; "
                                                   "git checkout $branch'")
                if status:
                    os.rmdir(project_path)
                    Log.error("Can not clone %s into %s" % (git_project_branch + " " + git_project_origin,
                                                            project_path))
                    exit(1)
                Log.debug("%s" % output)

        elif project_type == "svn":
            svn_project_url = as_conf.get_svn_project_url()
            svn_project_revision = as_conf.get_svn_project_revision()
            project_path = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/" + BasicConfig.LOCAL_PROJ_DIR
            if os.path.exists(project_path):
                Log.debug("The project folder exists. SKIPPING...")
                Log.info("Using project folder: %s" % project_path)
            else:
                os.mkdir(project_path)
                Log.debug("The project folder %s has been created." % project_path)
                Log.info("Checking out revision %s into %s" % (svn_project_revision + " " + svn_project_url,
                                                               project_path))
                (status, output) = getstatusoutput("cd " + project_path + "; svn checkout -r " + svn_project_revision +
                                                   " " + svn_project_url)
                if status:
                    os.rmdir(project_path)
                    Log.error("Can not check out revision %s into %s" % (svn_project_revision + " " + svn_project_url,
                                                                         project_path))
                    exit(1)
                Log.debug("%s" % output)

        elif project_type == "local":
            local_project_path = as_conf.get_local_project_path()
            project_path = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/" + BasicConfig.LOCAL_PROJ_DIR
            if os.path.exists(project_path):
                Log.debug("The project folder exists. SKIPPING...")
                Log.info("Using project folder: %s" % project_path)
            else:
                os.mkdir(project_path)
                Log.debug("The project folder %s has been created." % project_path)
                Log.info("Copying %s into %s" % (local_project_path, project_path))
                (status, output) = getstatusoutput("cp -R " + local_project_path + " " + project_path)
                if status:
                    os.rmdir(project_path)
                    Log.error("Can not copy %s into %s. Exiting..." % (local_project_path, project_path))
                    exit(1)
                Log.debug("%s" % output)

        if project_type != "none":
            # Check project configuration
            as_conf.check_proj()

        # Load parameters
        Log.info("Loading parameters...")
        parameters = as_conf.load_parameters()

        date_list = as_conf.get_date_list()
        if len(date_list) != len(set(date_list)):
            Log.error('There are repeated start dates!')
            exit(1)
        starting_chunk = as_conf.get_starting_chunk()
        num_chunks = as_conf.get_num_chunks()
        member_list = as_conf.get_member_list()
        if len(date_list) != len(set(date_list)):
            Log.error('There are repeated member names!')
            exit(1)
        rerun = as_conf.get_rerun()

        Log.info("\nCreating joblist...")
        job_list = JobList(expid)
        job_list.create(date_list, member_list, starting_chunk, num_chunks, parameters)
        if rerun == "true":
            chunk_list = Autosubmit._create_json(as_conf.get_chunk_list())
            job_list.rerun(chunk_list)

        pltfrm = as_conf.get_platform()
        if pltfrm == 'hector' or pltfrm == 'archer':
            job_list.update_shortened_names()

        Log.info("\nSaving joblist...")
        job_list.save()
        if not noplot:
            Log.info("\nPloting joblist...")
            monitor_exp = Monitor()
            monitor_exp.generate_output(expid, job_list.get_job_list(), 'pdf')

        Log.result("\nJob list created succesfully")
        Log.user_warning("Remember to MODIFY the MODEL config files!")

    @staticmethod
    def change_pkl(expid, root_name, save, final, lst, flt,
                   filter_chunks, filter_status, filter_section):
        BasicConfig.read()

        Log.set_file(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, BasicConfig.LOCAL_TMP_DIR,
                                  'change_pkl.log'))
        Log.debug('Exp ID: %', expid)
        l1 = cPickle.load(file(BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/pkl/" + root_name + "_" + expid +
                               ".pkl", 'r'))

        if flt:
            if filter_chunks:
                fc = filter_chunks
                Log.debug(fc)

                if fc == 'Any':
                    for job in l1.get_job_list():
                        job.status = Autosubmit._get_status(final)
                        Log.info("CHANGED: job: " + job.name + " status to: " + final)
                else:
                    data = json.loads(Autosubmit._create_json(fc))
                    # change localsetup and remotesetup
                    # [...]
                    for date in data['sds']:
                        for member in date['ms']:
                            jobname_ini = expid + "_" + str(date['sd']) + "_" + str(member['m']) + "_ini"
                            job = l1.get_job_by_name(jobname_ini)
                            job.status = Autosubmit._get_status(final)
                            Log.info("CHANGED: job: " + job.name + " status to: " + final)
                            # change also trans
                            jobname_trans = expid + "_" + str(date['sd']) + "_" + str(member['m']) + "_trans"
                            job = l1.get_job_by_name(jobname_trans)
                            job.status = Autosubmit._get_status(final)
                            Log.info("CHANGED: job: " + job.name + " status to: " + final)
                            # [...]
                            for chunk in member['cs']:
                                jobname_sim = expid + "_" + str(date['sd']) + "_" + str(member['m']) + "_" + str(
                                    chunk) + "_sim"
                                jobname_post = expid + "_" + str(date['sd']) + "_" + str(member['m']) + "_" + str(
                                    chunk) + "_post"
                                jobname_clean = expid + "_" + str(date['sd']) + "_" + str(member['m']) + "_" + str(
                                    chunk) + "_clean"
                                job = l1.get_job_by_name(jobname_sim)
                                job.status = Autosubmit._get_status(final)
                                Log.info("CHANGED: job: " + job.name + " status to: " + final)
                                job = l1.get_job_by_name(jobname_post)
                                job.status = Autosubmit._get_status(final)
                                Log.info("CHANGED: job: " + job.name + " status to: " + final)
                                job = l1.get_job_by_name(jobname_clean)
                                job.status = Autosubmit._get_status(final)
                                Log.info("CHANGED: job: " + job.name + " status to: " + final)

            if filter_status:
                fs = filter_status
                Log.debug(fs)

                if fs == 'Any':
                    for job in l1.get_job_list():
                        job.status = Autosubmit._get_status(final)
                        Log.info("CHANGED: job: " + job.name + " status to: " + final)
                else:
                    for job in l1.get_job_list():
                        if job.status == Autosubmit._get_status(fs):
                            job.status = Autosubmit._get_status(final)
                            Log.info("CHANGED: job: " + job.name + " status to: " + final)

            if filter_section:
                ft = filter_section
                Log.debug(ft)

                if ft == 'Any':
                    for job in l1.get_job_list():
                        job.status = Autosubmit._get_status(final)
                        Log.info("CHANGED: job: " + job.name + " status to: " + final)
                else:
                    for job in l1.get_job_list():
                        if job.section == ft:
                            job.status = Autosubmit._get_status(final)
                            Log.info("CHANGED: job: " + job.name + " status to: " + final)

        if lst:
            jobs = lst.split()

            if jobs == 'Any':
                for job in l1.get_job_list():
                    job.status = Autosubmit._get_status(final)
                    Log.info("CHANGED: job: " + job.name + " status to: " + final)
            else:
                for job in l1.get_job_list():
                    if job.name in jobs:
                        job.status = Autosubmit._get_status(final)
                        Log.info("CHANGED: job: " + job.name + " status to: " + final)

        sys.setrecursionlimit(50000)

        if save:
            l1.update_list()
            cPickle.dump(l1, file(BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/pkl/" + root_name + "_" + expid + ".pkl",
                                  'w'))
            Log.info("Saving JobList: " + BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/pkl/" + root_name + "_" + expid +
                     ".pkl")
        else:
            l1.update_list(False)
            Log.warning("Changes NOT saved to the JobList!!!!:  use -s option to save")

        monitor_exp = Monitor()

        monitor_exp.generate_output(expid, l1.get_job_list())

    @staticmethod
    def _user_yes_no_query(question):
        sys.stdout.write('%s [y/n]\n' % question)
        while True:
            try:
                return strtobool(raw_input().lower())
            except ValueError:
                sys.stdout.write('Please respond with \'y\' or \'n\'.\n')

    @staticmethod
    def _prepare_conf_files(exp_id, hpc, autosubmit_version, dummy):
        as_conf = AutosubmitConfig(exp_id)
        as_conf.set_version(autosubmit_version)
        as_conf.set_expid(exp_id)
        as_conf.set_local_root()
        as_conf.set_platform(hpc)
        as_conf.set_safetysleeptime(10)

        if dummy:
            content = file(as_conf.experiment_file).read()

            # Experiment
            content = content.replace(re.search('DATELIST =.*', content).group(0),
                                      "DATELIST = 20000101")
            content = content.replace(re.search('MEMBERS =.*', content).group(0),
                                      "MEMBERS = fc0")
            content = content.replace(re.search('CHUNKSIZE =.*', content).group(0),
                                      "CHUNKSIZE = 4")
            content = content.replace(re.search('NUMCHUNKS =.*', content).group(0),
                                      "NUMCHUNKS = 1")
            content = content.replace(re.search('PROJECT_TYPE =.*', content).group(0),
                                      "PROJECT_TYPE = none")

            file(as_conf.experiment_file, 'w').write(content)

    @staticmethod
    def _check_templates(as_conf):
        """Procedure to check autogeneration of templates given
        Autosubmit configuration.
        Returns True if all variables are set.
        If the parameters are not correctly replaced, the function returns
        False and the check fails.

        :param as_conf: Autosubmit configuration object
        :type: AutosubmitConf
        :retruns: bool
        """
        parameters = as_conf.load_parameters()
        joblist = JobList(parameters['EXPID'])
        joblist.create(parameters['DATELIST'].split(' '), parameters['MEMBERS'].split(' '), int(parameters['CHUNKINI']),
                       int(parameters['NUMCHUNKS']), parameters)
        out = joblist.check_scripts(as_conf)
        return out

    @staticmethod
    def _get_status(s):
        if s == 'READY':
            return Status.READY
        elif s == 'COMPLETED':
            return Status.COMPLETED
        elif s == 'WAITING':
            return Status.WAITING
        elif s == 'SUSPENDED':
            return Status.SUSPENDED
        elif s == 'FAILED':
            return Status.FAILED
        elif s == 'UNKNOWN':
            return Status.UNKNOWN

    @staticmethod
    def _get_members(out):
        count = 0
        data = []
        # noinspection PyUnusedLocal
        for element in out:
            if count % 2 == 0:
                ms = {'m': out[count], 'cs': Autosubmit._get_chunks(out[count + 1])}
                data.append(ms)
                count += 1
            else:
                count += 1

        return data

    @staticmethod
    def _get_chunks(out):
        data = []
        for element in out:
            if element.find("-") != -1:
                numbers = element.split("-")
                for count in range(int(numbers[0]), int(numbers[1]) + 1):
                    data.append(str(count))
            else:
                data.append(element)

        return data

    @staticmethod
    def _create_json(text):
        count = 0
        data = []
        # text = "[ 19601101 [ fc0 [1 2 3 4] fc1 [1] ] 16651101 [ fc0 [1-30 31 32] ] ]"

        out = nestedExpr('[', ']').parseString(text).asList()

        # noinspection PyUnusedLocal
        for element in out[0]:
            if count % 2 == 0:
                sd = {'sd': out[0][count], 'ms': Autosubmit._get_members(out[0][count + 1])}
                data.append(sd)
                count += 1
            else:
                count += 1

        sds = {'sds': data}
        result = json.dumps(sds)
        return result

    @staticmethod
    def test(expid, chunks, member, stardate, hpc, branch):
        testid = Autosubmit.expid('test', 'test experiment for {0}'.format(expid), expid, False)

        as_conf = AutosubmitConfig(testid)
        exp_parser = expdef_parser(as_conf.experiment_file)

        content = file(as_conf.experiment_file).read()
        if hpc is None:
            pass
        if member is None:
            member = random.choice(exp_parser.get('experiment', 'MEMBERS').split(' '))
        if stardate is None:
            stardate = random.choice(exp_parser.get('experiment', 'DATELIST').split(' '))
        # Experiment
        content = content.replace(re.search('DATELIST =.*', content).group(0),
                                  "DATELIST = " + stardate)
        content = content.replace(re.search('MEMBERS =.*', content).group(0),
                                  "MEMBERS = " + member)
        content = content.replace(re.search('NUMCHUNKS =.*', content).group(0),
                                  "NUMCHUNKS = " + chunks)
        content = content.replace(re.search('HPCARCH =.*', content).group(0),
                                  "HPCARCH = ithaca")
        if branch is not None:
            content = content.replace(re.search('PROJECT_BRANCH =.*', content).group(0),
                                      "PROJECT_BRANCH = " + branch)
            content = content.replace(re.search('PROJECT_REVISION =.*', content).group(0),
                                      "PROJECT_REVISION = " + branch)

        file(as_conf.experiment_file, 'w').write(content)

        Autosubmit.create(testid, False)
        if not Autosubmit.run_experiment(testid):
            exit(1)
        Autosubmit.delete(testid, True)




#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.

from os import path
from os import listdir
from shutil import rmtree
from commands import getstatusoutput

from autosubmit.config.basicConfig import BasicConfig
from autosubmit.config.log import Log


class AutosubmitGit:
    """Class to handle experiment git repository"""

    def __init__(self, expid):
        self._expid = expid

    def clean_git(self):
        """Function to clean space on BasicConfig.LOCAL_ROOT_DIR/git directory."""
        dirs = listdir(BasicConfig.LOCAL_ROOT_DIR + "/" + self._expid + "/" + BasicConfig.LOCAL_PROJ_DIR)
        if dirs:
            Log.debug("Checking git directories status...")
            for dirname in dirs:
                Log.debug("Directory: " + dirname)
                if path.isdir(BasicConfig.LOCAL_ROOT_DIR + "/" + self._expid + "/" + BasicConfig.LOCAL_PROJ_DIR +
                              "/" + dirname):
                    if path.isdir(BasicConfig.LOCAL_ROOT_DIR + "/" + self._expid + "/" + BasicConfig.LOCAL_PROJ_DIR +
                                  "/" + dirname + "/.git"):
                        (status, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" + self._expid +
                                                           "/" + BasicConfig.LOCAL_PROJ_DIR + "/" + dirname + "; " +
                                                           "git diff-index HEAD --")
                        if status == 0:
                            if output:
                                Log.info("Changes not commited detected... SKIPPING!")
                                Log.user_warning("Commit needed!")
                                exit(1)
                            else:
                                (status, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" +
                                                                   self._expid + "/" + BasicConfig.LOCAL_PROJ_DIR +
                                                                   "/" + dirname + "; " +
                                                                   "git log --branches --not --remotes")
                                if output:
                                    Log.info("Changes not pushed detected... SKIPPING!")
                                    Log.user_warning("Synchronization needed!")
                                    exit(1)
                                else:
                                    Log.debug("Ready to clean...")
                                    Log.debug("Cloning: 'git clone --bare " + dirname + " " + dirname + ".git' ...")
                                    # noinspection PyUnusedLocal
                                    (status, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" +
                                                                       self._expid + "/" + BasicConfig.LOCAL_PROJ_DIR +
                                                                       "/" + "; " + "git clone --bare " + dirname +
                                                                       " " + dirname + ".git")
                                    Log.debug("Removing: " + dirname)
                                    rmtree(BasicConfig.LOCAL_ROOT_DIR + "/" + self._expid + "/" +
                                           BasicConfig.LOCAL_PROJ_DIR + "/" + dirname)
                                    Log.debug(dirname + " directory clean!")
                                    Log.user_warning("Further runs will require 'git clone " + dirname + ".git " +
                                                     dirname + "' ...")
                        else:
                            Log.error("Failed to retrieve git info...")
                            exit(1)
                    else:
                        Log.debug("Not a git repository... SKIPPING!")
                else:
                    Log.debug("Not a directory... SKIPPING!")
        return



#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
from ConfigParser import SafeConfigParser
import os

from autosubmit.config.log import Log


class BasicConfig:

    DB_DIR = '~/autosubmit'
    DB_FILE = 'autosubmit.db'
    DB_PATH = DB_DIR + "/" + DB_FILE
    LOCAL_ROOT_DIR = '~/autosubmit'
    LOCAL_TMP_DIR = 'tmp'
    LOCAL_PROJ_DIR = 'proj'

    @staticmethod
    def update_config():
        # Just one needed for the moment.
        BasicConfig.DB_PATH = os.path.join(BasicConfig.DB_DIR, BasicConfig.DB_FILE)

    @staticmethod
    def __read_file_config(file_path):
        if not os.path.isfile(file_path):
            return
        Log.debug('Reading config from ' + file_path)
        parser = SafeConfigParser()
        parser.optionxform = str
        parser.read(file_path)

        if parser.has_option('database', 'path'):
            BasicConfig.DB_DIR = parser.get('database', 'path')
        if parser.has_option('local', 'path'):
            BasicConfig.LOCAL_ROOT_DIR = parser.get('local', 'path')

    @staticmethod
    def read():
        filename = '.autosubmitrc'

        BasicConfig.__read_file_config(os.path.join('/etc', filename))
        BasicConfig.__read_file_config(os.path.join(os.path.expanduser('~'), filename))
        BasicConfig.__read_file_config(os.path.join('.', filename))

        BasicConfig.update_config()
        return


#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.

import datetime
import time
import calendar
from dateutil.relativedelta import *

from autosubmit.config.log import Log


"""In this python script there are tools to manipulate the dates and make mathematical operations between them """


def add_time(string_date, total_size, chunk_unit, cal):
    if chunk_unit == 'year':
        return add_years(string_date, total_size)
    elif chunk_unit == 'month':
        return add_months(string_date, total_size, cal)
    elif chunk_unit == 'day':
        return add_days(string_date, total_size, cal)
    elif chunk_unit == 'hour':
        return add_hours(string_date, total_size, cal)
    else:
        Log.critical('Chunk unit not valid: {0}'.format(chunk_unit))


def add_years(string_date, number_of_years):
    date = time.strptime(string_date, '%Y%m%d')
    delta = relativedelta(years=number_of_years)
    result = datetime.date(date.tm_year, date.tm_mon, date.tm_mday) + delta
    return result


def add_months(string_date, number_of_months, cal):
    date = time.strptime(string_date, '%Y%m%d')
    delta = relativedelta(months=number_of_months)
    result = datetime.date(date.tm_year, date.tm_mon, date.tm_mday) + delta
    if cal == 'noleap':
        if result.month == 2 and result.day == 29:
            result = datetime.date(result.year, 2, 28)
    return result


def add_days(string_date, number_of_days, cal):
    date = time.strptime(string_date, '%Y%m%d')
    delta = relativedelta(days=number_of_days)
    result = datetime.date(date.tm_year, date.tm_mon, date.tm_mday) + delta
    if cal == 'noleap':
        year = date.tm_year
        if date.tm_mon > 2:
            year += 1

        while year <= result.year:
            if calendar.isleap(year):
                if result.year == year and result < datetime.date(year, 2, 29):
                    year += 1
                    continue
                result += relativedelta(days=1)
            year += 1
    return result


def add_hours(string_date, number_of_months, cal):
    date = time.strptime(string_date, '%Y%m%d')
    delta = relativedelta(hours=number_of_months)
    result = datetime.date(date.tm_year, date.tm_mon, date.tm_mday) + delta
    if cal == 'noleap':
        year = date.tm_year
        if date.tm_mon > 2:
            year += 1

        while year <= result.year:
            if calendar.isleap(year):
                if result.year == year and result < datetime.date(year, 2, 29):
                    year += 1
                    continue
                result += relativedelta(days=1)
            year += 1
    return result


def subs_dates(start_date, end_date, cal):
    start = time.strptime(start_date, '%Y%m%d')
    end = time.strptime(end_date, '%Y%m%d')
    start_datetime = datetime.date(start.tm_year, start.tm_mon, start.tm_mday)
    end_datetime = datetime.date(end.tm_year, end.tm_mon, end.tm_mday)
    result = end_datetime - start_datetime
    if cal == 'noleap':
        year = start_datetime.year
        if start_datetime.month > 2:
            year += 1

        while year <= end_datetime.year:
            if calendar.isleap(year):
                if end_datetime.year == year and end_datetime < datetime.date(year, 2, 29):
                    year += 1
                    continue
                result -= datetime.timedelta(days=1)
            year += 1
    return result.days


def chunk_start_date(string_date, chunk, chunk_length, chunk_unit, cal):
    chunk_1 = chunk - 1
    total_months = chunk_1 * chunk_length
    result = add_time(string_date, total_months, chunk_unit, cal)
    start_date = "%s%02d%02d" % (result.year, result.month, result.day)
    return start_date


def chunk_end_date(start_date, chunk_length, chunk_unit, cal):
    result = add_time(start_date, chunk_length, chunk_unit, cal)
    end_date = "%s%02d%02d" % (result.year, result.month, result.day)
    return end_date


def running_days(start_date, end_date, cal):
    return subs_dates(start_date, end_date, cal)


def previous_days(string_date, start_date, cal):
    return subs_dates(string_date, start_date, cal)


def previous_day(string_date, cal):
    date_1 = add_days(string_date, -1, cal)
    string_date_1 = "%s%02d%02d" % (date_1.year, date_1.month, date_1.day)
    return string_date_1


def chunk_start_month(string_date):
    date = time.strptime(string_date, '%Y%m%d')
    result = date.tm_mon
    return result


def chunk_start_year(string_date):
    date = time.strptime(string_date, '%Y%m%d')
    result = date.tm_year
    return result


####################
# Main Program
####################
def main():
    string_date = '19600201'
    cal = 'noleap'
    start_date = chunk_start_date(string_date, 1, 28, 'day', cal)
    Log.info(start_date)
    end_date = chunk_end_date(start_date, 28, 'day', cal)
    Log.info(end_date)
    Log.info(running_days(start_date, end_date, cal))
    Log.info(running_days(string_date, end_date, cal))
    Log.info(previous_days(string_date, start_date, cal))
    Log.info("year: {0}".format(chunk_start_year(string_date)))
    Log.info("month: {0}".format(chunk_start_month(string_date)))
    Log.info("yesterday: %s " % previous_day(string_date, cal))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.

"""Script for handling experiment monitoring"""
import os
import sys

scriptdir = os.path.abspath(os.path.dirname(sys.argv[0]))
assert sys.path[0] == scriptdir
sys.path[0] = os.path.normpath(os.path.join(scriptdir, os.pardir))

# noinspection PyUnresolvedReferences
from autosubmit.Autosubmit import Autosubmit


def main():
    Autosubmit.parse_args()

if __name__ == "__main__":
    main()


#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import textwrap

from autosubmit.queue.hpcqueue import HPCQueue
from autosubmit.config.basicConfig import BasicConfig
from autosubmit.config.log import Log


class EcQueue(HPCQueue):

    def __init__(self, expid, scheduler):
        HPCQueue.__init__(self)
        self._host = ""
        self.scratch = ""
        self.project = ""
        self.user = ""
        if scheduler == 'pbs':
            self._header = EcCcaHeader()
        elif scheduler == 'loadleveler':
            self._header = EcHeader()
        else:
            Log.error('ecaccess scheduler {0} not supported'.format(scheduler))
            exit(1)
        self.expid = expid
        self.job_status = dict()
        self.job_status['COMPLETED'] = ['DONE']
        self.job_status['RUNNING'] = ['EXEC']
        self.job_status['QUEUING'] = ['INIT', 'RETR', 'STDBY', 'WAIT']
        self.job_status['FAILED'] = ['STOP']
        self._pathdir = "\$HOME/LOG_" + self.expid
        self.update_cmds()

    def update_cmds(self):
        self.remote_log_dir = (self.scratch + "/" + self.project + "/" + self.user + "/" + self.expid + "/LOG_" +
                               self.expid)
        self.cancel_cmd = "eceaccess-job-delete"
        self._checkjob_cmd = "ecaccess-job-list "
        self._checkhost_cmd = "ecaccess-certificate-list"
        self._submit_cmd = ("ecaccess-job-submit -queueName " + self._host + " " + BasicConfig.LOCAL_ROOT_DIR + "/" +
                            self.expid + "/tmp/")
        self.put_cmd = "ecaccess-file-put"
        self.get_cmd = "ecaccess-file-get"
        self.mkdir_cmd = ("ecaccess-file-mkdir " + self._host + ":" + self.scratch + "/" + self.project + "/" +
                          self.user + "/" + self.expid + "; " + "ecaccess-file-mkdir " + self._host + ":" +
                          self.remote_log_dir)

    def get_checkhost_cmd(self):
        return self._checkhost_cmd

    def get_remote_log_dir(self):
        return self.remote_log_dir

    def get_mkdir_cmd(self):
        return self.mkdir_cmd

    def parse_job_output(self, output):
        job_state = output.split('\n')[7].split()[1]
        return job_state

    def get_submitted_job_id(self, output):
        return output

    def jobs_in_queue(self, output):
        Log.debug(output)
        return output.split()

    def get_checkjob_cmd(self, job_id):
        return self._checkjob_cmd + str(job_id)

    def get_submit_cmd(self, job_script):
        return self._submit_cmd + job_script


class EcHeader:
    """Class to handle the ECMWF headers of a job"""

    SERIAL = textwrap.dedent("""
            #!/bin/ksh
            ###############################################################################
            #                   %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #@ shell            = /usr/bin/ksh
            #@ class            = ns
            #@ job_type         = serial
            #@ job_name         = %JOBNAME%
            #@ output           = %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%/$(job_name).$(jobid).out
            #@ error            = %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%/$(job_name).$(jobid).err
            #@ notification     = error
            #@ resources        = ConsumableCpus(1) ConsumableMemory(1200mb)
            #@ wall_clock_limit = %WALLCLOCK%:00
            #@ queue
            #
            ###############################################################################
            """)

    PARALLEL = textwrap.dedent("""\
            #!/bin/ksh
            ###############################################################################
            #                   %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #@ shell            = /usr/bin/ksh
            #@ class            = np
            #@ job_type         = parallel
            #@ job_name         = %JOBNAME%
            #@ output           = %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%/$(job_name).$(jobid).out
            #@ error            = %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%/$(job_name).$(jobid).err
            #@ notification     = error
            #@ resources        = ConsumableCpus(1) ConsumableMemory(1200mb)
            #@ ec_smt           = no
            #@ total_tasks      = %NUMPROC%
            #@ wall_clock_limit = %WALLCLOCK%:00
            #@ queue
            #
            ###############################################################################
            """)


class EcCcaHeader:
    """Class to handle the ECMWF headers of a job"""

    SERIAL = textwrap.dedent("""
             #!/bin/bash
             ###############################################################################
             #                   %TASKTYPE% %EXPID% EXPERIMENT
             ###############################################################################
             #
             #PBS -N %JOBNAME%
             #PBS -q ns
             #PBS -l walltime=%WALLCLOCK_SETUP%:00
             #PBS -l EC_billing_account=%HPCPROJ%
             #
             ###############################################################################

            """)

    PARALLEL = textwrap.dedent("""\
             #!/bin/bash
             ###############################################################################
             #                   %TASKTYPE% %EXPID% EXPERIMENT
             ###############################################################################
             #
             #PBS -N %JOBNAME%
             #PBS -q np
             #PBS -l EC_total_tasks=%NUMPROC_SIM%
             #PBS -l EC_threads_per_task=%NUMTHREAD_SIM%
             #PBS -l EC_tasks_per_node=%NUMTASK_SIM%
             #PBS -l walltime=%WALLCLOCK_SIM%:00
             #PBS -l EC_billing_account=%HPCPROJ%
             #
             ###############################################################################
            """)


#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
from ConfigParser import SafeConfigParser
import os
import platform
import re
from os import listdir
from commands import getstatusoutput

from autosubmit.config.log import Log
from autosubmit.config.config_parser import config_parser
from autosubmit.config.config_parser import expdef_parser
from autosubmit.config.config_parser import projdef_parser
from autosubmit.config.basicConfig import BasicConfig

from autosubmit.queue.psqueue import PsQueue
from autosubmit.queue.lsfqueue import LsfQueue
from autosubmit.queue.pbsqueue import PBSQueue
from autosubmit.queue.sgequeue import SgeQueue
from autosubmit.queue.ecqueue import EcQueue


class AutosubmitConfig:
    """Class to handle experiment configuration coming from file or database"""

    def __init__(self, expid):
        self._conf_parser_file = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/conf/" + "autosubmit_" + expid + ".conf"
        self._exp_parser_file = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/conf/" + "expdef_" + expid + ".conf"

    @property
    def experiment_file(self):
        return self._exp_parser_file

    def get_project_dir(self):
        dir_templates = os.path.join(BasicConfig.LOCAL_ROOT_DIR, self.get_expid(), BasicConfig.LOCAL_PROJ_DIR)
        # Getting project name for each type of project
        if self.get_project_type().lower() == "local":
            dir_templates = os.path.join(dir_templates, os.path.split(self.get_local_project_path())[1])
        elif self.get_project_type().lower() == "git":
            dir_templates = self.get_git_project_origin().split('.')[-2]
        return dir_templates

    def check_conf(self):
        self._conf_parser = config_parser(self._conf_parser_file)
        self._exp_parser = expdef_parser(self._exp_parser_file)

    def check_proj(self):
        self._proj_parser_file = self.get_file_project_conf()
        if self._proj_parser_file == '':
            self._proj_parser = None
        else:
            self._proj_parser_file = os.path.join(self.get_project_dir(), self._proj_parser_file)
            self._proj_parser = projdef_parser(self._proj_parser_file)

    def reload(self):
        self._conf_parser = config_parser(self._conf_parser_file)
        self._exp_parser_file = self._conf_parser.get('config', 'EXPDEFFILE')
        self._exp_parser = expdef_parser(self._exp_parser_file)
        project_type = self.get_project_type()
        if project_type != "none":
            self.check_proj()

    def load_parameters(self):
        expdef = []
        for section in self._exp_parser.sections():
            expdef += self._exp_parser.items(section)

        for section in self._conf_parser.sections():
            expdef += self._conf_parser.items(section)

        parameters = dict()
        for item in expdef:
            parameters[item[0]] = item[1]

        project_type = self.get_project_type()
        if project_type != "none" and self._proj_parser is not None:
            # Load project parameters
            Log.debug("Loading project parameters...")
            parameters2 = parameters.copy()
            parameters2.update(self.load_project_parameters())
            parameters = parameters2

        return parameters

    def load_project_parameters(self):
        projdef = []
        for section in self._proj_parser.sections():
            projdef += self._proj_parser.items(section)

        parameters = dict()
        for item in projdef:
            parameters[item[0]] = item[1]

        return parameters

    @staticmethod
    def print_parameters(title, parameters):
        """Prints the parameters table in a tabular mode"""
        Log.info(title)
        Log.info("----------------------")
        Log.info("{0:<{col1}}| {1:<{col2}}".format("-- Parameter --", "-- Value --", col1=15, col2=15))
        for i in parameters:
            Log.info("{0:<{col1}}| {1:<{col2}}".format(i[0], i[1], col1=15, col2=15))
        Log.info("")

    def check_parameters(self):
        """Function to check configuration of Autosubmit.
        Returns True if all variables are set.
        If some parameter do not exist, the function returns False.

        :retruns: bool
        """
        result = True

        for section in self._conf_parser.sections():
            self.print_parameters("AUTOSUBMIT PARAMETERS - " + section, self._conf_parser.items(section))
            if "" in [item[1] for item in self._conf_parser.items(section)]:
                result = False
        for section in self._exp_parser.sections():
            self.print_parameters("EXPERIMENT PARAMETERS - " + section, self._exp_parser.items(section))
            if "" in [item[1] for item in self._exp_parser.items(section)]:
                result = False

        project_type = self.get_project_type()
        if project_type != "none" and self._proj_parser is not None:
            for section in self._proj_parser.sections():
                self.print_parameters("PROJECT PARAMETERS - " + section, self._proj_parser.items(section))
                if "" in [item[1] for item in self._proj_parser.items(section)]:
                    result = False

        return result

    def get_expid(self):
        return self._conf_parser.get('config', 'EXPID')

    def set_expid(self, exp_id):
        # Autosubmit conf
        content = file(self._conf_parser_file).read()
        if re.search('EXPID =.*', content):
            content = content.replace(re.search('EXPID =.*', content).group(0), "EXPID = " + exp_id)
        file(self._conf_parser_file, 'w').write(content)
        # Experiment conf
        content = file(self._exp_parser_file).read()
        if re.search('EXPID =.*', content):
            content = content.replace(re.search('EXPID =.*', content).group(0), "EXPID = " + exp_id)
        file(self._exp_parser_file, 'w').write(content)

    def get_project_type(self):
        return self._exp_parser.get('project', 'PROJECT_TYPE').lower()

    def get_file_project_conf(self):
        return self._exp_parser.get('project_files', 'FILE_PROJECT_CONF').lower()

    def get_git_project_origin(self):
        return self._exp_parser.get('git', 'PROJECT_ORIGIN').lower()

    def get_git_project_branch(self):
        return self._exp_parser.get('git', 'PROJECT_BRANCH').lower()

    def get_git_project_commit(self):
        return self._exp_parser.get('git', 'PROJECT_COMMIT').lower()

    def set_git_project_commit(self):
        """Function to register in the configuration the commit SHA of the git project version."""
        save = False
        project_branch_sha = None
        project_name = listdir(BasicConfig.LOCAL_ROOT_DIR + "/" + self.get_expid() + "/" +
                               BasicConfig.LOCAL_PROJ_DIR)[0]
        (status1, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" + self.get_expid() + "/" +
                                            BasicConfig.LOCAL_PROJ_DIR + "/" + project_name)
        (status2, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" + self.get_expid() + "/" +
                                            BasicConfig.LOCAL_PROJ_DIR + "/" + project_name + "; " +
                                            "git rev-parse --abbrev-ref HEAD")
        if status1 == 0 and status2 == 0:
            project_branch = output
            Log.debug("Project branch is: " + project_branch)

            (status1, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" + self.get_expid() + "/" +
                                                BasicConfig.LOCAL_PROJ_DIR + "/" + project_name)
            (status2, output) = getstatusoutput("cd " + BasicConfig.LOCAL_ROOT_DIR + "/" + self.get_expid() + "/" +
                                                BasicConfig.LOCAL_PROJ_DIR + "/" + project_name + "; " +
                                                "git rev-parse HEAD")
            if status1 == 0 and status2 == 0:
                project_sha = output
                save = True
                Log.debug("Project commit SHA is: " + project_sha)
                project_branch_sha = project_branch + " " + project_sha
            else:
                Log.critical("Failed to retrieve project commit SHA...")

        else:
            Log.critical("Failed to retrieve project branch...")

            # register changes
        if save:
            content = file(self._exp_parser_file).read()
            if re.search('PROJECT_COMMIT =.*', content):
                content = content.replace(re.search('PROJECT_COMMIT =.*', content).group(0),
                                          "PROJECT_COMMIT = " + project_branch_sha)
            file(self._exp_parser_file, 'w').write(content)
            Log.debug("Project commit SHA succesfully registered to the configuration file.")
        else:
            Log.critical("Changes NOT registered to the configuration file...")

    def get_svn_project_url(self):
        return self._exp_parser.get('svn', 'PROJECT_URL').lower()

    def get_svn_project_revision(self):
        return self._exp_parser.get('svn', 'PROJECT_REVISION').lower()

    def get_local_project_path(self):
        return self._exp_parser.get('local', 'PROJECT_PATH').lower()

    def get_date_list(self):
        return self._exp_parser.get('experiment', 'DATELIST').split(' ')

    def get_starting_chunk(self):
        return int(self._exp_parser.get('experiment', 'CHUNKINI'))

    def get_num_chunks(self):
        return int(self._exp_parser.get('experiment', 'NUMCHUNKS'))

    def get_member_list(self):
        return self._exp_parser.get('experiment', 'MEMBERS').split(' ')

    def get_rerun(self):
        return self._exp_parser.get('rerun', 'RERUN').lower()

    def get_chunk_list(self):
        return self._exp_parser.get('rerun', 'CHUNKLIST')

    def get_platform(self):
        return self._exp_parser.get('experiment', 'HPCARCH').lower()

    def set_platform(self, hpc):
        content = file(self._exp_parser_file).read()
        if re.search('HPCARCH =.*', content):
            content = content.replace(re.search('HPCARCH =.*', content).group(0), "HPCARCH = " + hpc)
        file(self._exp_parser_file, 'w').write(content)

    def set_version(self, autosubmit_version):
        content = file(self._conf_parser_file).read()
        if re.search('AUTOSUBMIT_VERSION =.*', content):
            content = content.replace(re.search('AUTOSUBMIT_VERSION =.*', content).group(0),
                                      "AUTOSUBMIT_VERSION = " + autosubmit_version)
        file(self._conf_parser_file, 'w').write(content)

    def set_local_root(self):
        content = file(self._conf_parser_file).read()
        if re.search('AUTOSUBMIT_LOCAL_ROOT =.*', content):
            content = content.replace(re.search('AUTOSUBMIT_LOCAL_ROOT =.*', content).group(0),
                                      "AUTOSUBMIT_LOCAL_ROOT = " + BasicConfig.LOCAL_ROOT_DIR)
        file(self._conf_parser_file, 'w').write(content)

    def get_total_jobs(self):
        return int(self._conf_parser.get('config', 'TOTALJOBS'))

    def get_max_waiting_jobs(self):
        return int(self._conf_parser.get('config', 'MAXWAITINGJOBS'))

    def get_safetysleeptime(self):
        return int(self._conf_parser.get('config', 'SAFETYSLEEPTIME'))

    def set_safetysleeptime(self, sleep_time):
        content = file(self._conf_parser_file).read()
        content = content.replace(re.search('SAFETYSLEEPTIME =.*', content).group(0),
                                  "SAFETYSLEEPTIME = %d" % sleep_time)
        file(self._conf_parser_file, 'w').write(content)

    def get_retrials(self):
        return int(self._conf_parser.get('config', 'RETRIALS'))

    @staticmethod
    def read_queues_conf(expid):
        parser = SafeConfigParser()
        parser.optionxform = str
        parser.read(os.path.join(BasicConfig.LOCAL_ROOT_DIR, expid, 'conf', "queues_" + expid + ".conf"))

        queues = dict()
        local_queue = PsQueue(expid)
        local_queue.name = 'local'
        local_queue.type = 'ps'
        local_queue.set_host(platform.node())
        local_queue.set_scratch(BasicConfig.LOCAL_ROOT_DIR)
        local_queue.set_project(expid)
        local_queue.set_user(BasicConfig.LOCAL_TMP_DIR)
        local_queue.update_cmds()

        queues['local'] = local_queue
        for section in parser.sections():
            queue_type = AutosubmitConfig.get_option(parser, section, 'TYPE', '').lower()
            queue_version = AutosubmitConfig.get_option(parser, section, 'VERSION', None)
            queue = None
            if queue_type == 'pbs':
                queue = PBSQueue(expid, queue_version)
            elif queue_type == 'sge':
                queue = SgeQueue(expid)
            elif queue_type == 'ps':
                queue = PsQueue(expid)
            elif queue_type == 'lsf':
                queue = LsfQueue(expid)
            elif queue_type == 'ecaccess':
                queue = EcQueue(expid, queue_version)
            elif queue_type == '':
                Log.error("Queue type not specified".format(queue_type))
                exit(1)
            else:
                Log.error("Queue type {0} not defined".format(queue_type))
                exit(1)

            queue.type = queue_type
            if AutosubmitConfig.get_option(parser, section, 'ADD_PROJECT_TO_HOST', '').lower() == 'true':
                host = '{0}-{1}'.format(AutosubmitConfig.get_option(parser, section, 'HOST', None),
                                        AutosubmitConfig.get_option(parser, section, 'PROJECT', None))
            else:
                host = AutosubmitConfig.get_option(parser, section, 'HOST', None)
            queue.set_host(host)
            queue.set_project(AutosubmitConfig.get_option(parser, section, 'PROJECT', None))
            queue.set_user(AutosubmitConfig.get_option(parser, section, 'USER', None))
            queue.set_scratch(AutosubmitConfig.get_option(parser, section, 'SCRATCH_DIR', None))
            queue.name = section.lower()
            queue.update_cmds()
            queues[section.lower()] = queue

        for section in parser.sections():
            if parser.has_option(section, 'SERIAL_QUEUE'):
                queues[section.lower()].set_serial_queue(queues[AutosubmitConfig.get_option(parser, section,
                                                                                            'SERIAL_QUEUE',
                                                                                            None).lower()])

        return queues

    @staticmethod
    def get_option(parser, section, option, default):
        if parser.has_option(section, option):
            return parser.get(section, option)
        else:
            return default





#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import os
import sys
import sqlite3
import string

from autosubmit.config.log import Log
from autosubmit.config.basicConfig import BasicConfig


def create_db(qry):
    (conn, cursor) = open_conn()
    try:
        cursor.execute(qry)
    except sqlite3.Error:
        close_conn(conn, cursor)
        Log.error('The database can not be created.' + 'DB file:' + BasicConfig.DB_PATH)
        sys.exit(1)

    conn.commit()
    close_conn(conn, cursor)
    return


def set_experiment(name, description):
    check_db()
    name = check_name(name)

    (conn, cursor) = open_conn()
    try:
        cursor.execute('insert into experiment (name, description) values (:name, :description)',
                       {'name': name, 'description': description})
    except sqlite3.IntegrityError as e:
        close_conn(conn, cursor)
        Log.error('Could not register experiment: {0}'.format(e))
        sys.exit(1)

    conn.commit()
    close_conn(conn, cursor)
    return


def check_experiment_exists(name):
    """
    Checks if exist an experiment with the given name.
    :param name: Experiment name
    :return: If experiment exists returns true, if not returns false
    """
    check_db()
    name = check_name(name)

    (conn, cursor) = open_conn()

    # SQLite always return a unicode object, but we can change this
    # behaviour with the next sentence
    conn.text_factory = str
    cursor.execute('select name from experiment where name=:name', {'name': name})
    row = cursor.fetchone()
    close_conn(conn, cursor)
    if row is None:
        Log.error('The experiment name %s does not exist yet!!!' % name)
        return False
    return True


def new_experiment(hpc, description):
    last_exp_name = last_name(hpc)
    if last_exp_name == 'empty':
        if hpc == 'test':
            new_name = 'test000'
        else:
            new_name = hpc[0]+'000'
    else:
        new_name = next_name(last_exp_name)
    set_experiment(new_name, description)
    Log.info('The new experiment "%s" has been registered.' % new_name)
    return new_name


def copy_experiment(name, hpc, description):
    if not check_experiment_exists(name):
        exit(1)
    new_name = new_experiment(hpc, description)
    return new_name


def base36encode(number, alphabet=string.digits + string.ascii_lowercase):
    """Convert positive integer to a base36 string."""
    if not isinstance(number, (int, long)):
        raise TypeError('number must be an integer')

    # Special case for zero
    if number == 0:
        return '0'

    base36 = ''

    sign = ''
    if number < 0:
        sign = '-'
        number = - number

    while number > 0:
        number, i = divmod(number, len(alphabet))
        base36 = alphabet[i] + base36

    return sign + base36


def base36decode(number):
    return int(number, 36)


def next_name(name):
    name = check_name(name)
    # Convert the name to base 36 in number add 1 and then encode it
    return base36encode(base36decode(name) + 1)


def last_name(hpc):
    check_db()
    (conn, cursor) = open_conn()
    conn.text_factory = str
    if hpc == 'test':
        hpc_name = 'test'
    else:
        hpc_name = hpc[0]
    hpc_name += '___'
    cursor.execute('select name '
                   'from experiment '
                   'where rowid=(select max(rowid) from experiment where name LIKE "' + hpc_name + '")')
    row = cursor.fetchone()
    if row is None:
        row = ('empty', )
    close_conn(conn, cursor)
    return row[0]


def delete_experiment(name):
    check_db()
    name = check_name(name)
    (conn, cursor) = open_conn()
    cursor.execute('delete from experiment '
                   'where name=:name', {'name': name})
    row = cursor.fetchone()
    if row is None:
        Log.debug('The experiment %s has been deleted!!!' % name)
    close_conn(conn, cursor)
    return


def check_name(name):
    name = name.lower()
    if len(name) < 4 or not name.isalnum():
        Log.error("So sorry, but the name must have at least 4 alphanumeric chars!!!")
        sys.exit(1)
    return name


def check_db():
    if not os.path.exists(BasicConfig.DB_PATH):
        Log.error('Some problem has happened...check the database file.' + 'DB file:' + BasicConfig.DB_PATH)
        sys.exit(1)
    return


def open_conn():
    conn = sqlite3.connect(BasicConfig.DB_PATH)
    cursor = conn.cursor()
    return conn, cursor


def close_conn(conn, cursor):
    conn.commit()
    cursor.close()
    conn.close()
    return




#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import sys
import re
from os import path
from ConfigParser import SafeConfigParser

from pyparsing import nestedExpr

from autosubmit.config.log import Log

invalid_values = False


def check_values(key, value, valid_values):
    global invalid_values

    if value.lower() not in valid_values:
        Log.error("Invalid value %s: %s", key, value)
        invalid_values = True


def check_regex(key, value, regex):
    global invalid_values

    prog = re.compile(regex)

    if not prog.match(value.lower()):
        Log.error("Invalid value %s: %s" % (key, value))
        invalid_values = True


def check_json(key, value):
    global invalid_values

    # noinspection PyBroadException
    try:
        nestedExpr('[', ']').parseString(value).asList()
    except:
        Log.error("Invalid value %s: %s" % (key, value))
        invalid_values = True


def config_parser(filename):
    loglevel = ['debug', 'info', 'warning', 'error', 'critical']

    # check file existance
    if not path.isfile(filename):
        Log.error("File does not exist: " + filename)
        sys.exit(1)

    # load values
    parser = SafeConfigParser()
    parser.optionxform = str
    parser.read(filename)

    check_values('LOGLEVEL', parser.get('config', 'LOGLEVEL'), loglevel)

    if invalid_values:
        Log.error("Invalid Autosubmit config file")
        sys.exit(1)
    else:
        Log.debug("Autosubmit config file OK")

    return parser


def expdef_parser(filename):
    startdate = "(\s*[0-9]{4}[0-9]{2}[0-9]{2}\s*)+$"
    chunkini = "\s*\d+\s*$"
    numchunks = "\s*\d+\s*$"
    chunksize = "\s*\d+\s*$"
    members = "(\s*fc\d+\s*)+$"
    rerun = "\s*(true|false)\s*$"
    projecttype = ['git', 'svn', 'local', 'none']

    # option that must be in config file and has no default value
    mandatory_opt = ['EXPID']

    # check file existance
    if not path.isfile(filename):
        Log.critical("File does not exist: " + filename)
        sys.exit(1)

    # load values
    parser = SafeConfigParser()
    parser.optionxform = str
    parser.read(filename)

    # check which options of the mandatory one are not in config file
    missing = list(set(mandatory_opt).difference(parser.options('experiment')))
    if missing:
        Log.critical("Missing options: " + ','.join(missing))
        sys.exit(1)

    # check create_exp.py variables
    check_regex('DATELIST', parser.get('experiment', 'DATELIST'), startdate)
    check_regex('MEMBERS', parser.get('experiment', 'MEMBERS'), members)
    check_regex('CHUNKINI', parser.get('experiment', 'CHUNKINI'), chunkini)
    check_regex('NUMCHUNKS', parser.get('experiment', 'NUMCHUNKS'), numchunks)
    check_regex('CHUNKSIZE', parser.get('experiment', 'CHUNKSIZE'), chunksize)
    check_regex('RERUN', parser.get('rerun', 'RERUN'), rerun)
    if parser.get('rerun', 'RERUN') == "TRUE":
        check_json('CHUNKLIST', parser.get('rerun', 'CHUNKLIST'))
    check_values('PROJECT_TYPE', parser.get('project', 'PROJECT_TYPE'), projecttype)
    # if (parser.get('project', 'PROJECT_TYPE') == "git"):
    # check_regex('PROJECT_ORIGIN', parser.get('git', 'PROJECT_ORIGIN'), gitorigin)

    if invalid_values:
        Log.error("Invalid experiment config file")
        sys.exit(1)
    else:
        Log.debug("Experiment config file OK")

    return parser


def projdef_parser(filename):
    # check file existance
    if not path.isfile(filename):
        Log.critical("File does not exist: " + filename)
        sys.exit()

    # load values
    parser = SafeConfigParser()
    parser.optionxform = str
    parser.read(filename)

    if invalid_values:
        Log.error("Invalid project config file")
        sys.exit()
    else:
        Log.debug("Project config file OK")

    return parser


#
# ####################
# # Main Program
# ####################
# def main():
# if len(sys.argv) != 2:
#         print "Error missing config file"
#     else:
#         autosubmit_conf_parser(sys.argv[1])
#
#
# if __name__ == "__main__":
#     main()



#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
from ConfigParser import SafeConfigParser
import json

import os
import pickle
from time import localtime, strftime
from sys import setrecursionlimit
from shutil import move

from job_common import Status
from autosubmit.job.job import Job
from autosubmit.config.basicConfig import BasicConfig
from autosubmit.config.log import Log


class JobList:
    def __init__(self, expid):
        self._pkl_path = BasicConfig.LOCAL_ROOT_DIR + "/" + expid + "/pkl/"
        self._update_file = "updated_list_" + expid + ".txt"
        self._failed_file = "failed_job_list_" + expid + ".pkl"
        self._job_list_file = "job_list_" + expid + ".pkl"
        self._job_list = list()
        self._expid = expid
        self._stat_val = Status()
        self._parameters = []

    @property
    def expid(self):
        return self._expid

    def create(self, date_list, member_list, starting_chunk, num_chunks, parameters):
        self._parameters = parameters

        parser = SafeConfigParser()
        parser.optionxform = str
        parser.read(os.path.join(BasicConfig.LOCAL_ROOT_DIR, self._expid, 'conf', "jobs_" + self._expid + ".conf"))

        chunk_list = range(starting_chunk, starting_chunk + num_chunks)

        self._date_list = date_list
        self._member_list = member_list
        self._chunk_list = chunk_list

        dic_jobs = DicJobs(self, parser, date_list, member_list, chunk_list)
        self._dic_jobs = dic_jobs
        priority = 0

        Log.info("Creating jobs...")
        for section in parser.sections():
            Log.debug("Creating {0} jobs".format(section))
            dic_jobs.read_section(section, priority)
            priority += 1

        Log.info("Adding dependencies...")
        for section in parser.sections():
            Log.debug("Adding dependencies for {0} jobs".format(section))
            if not parser.has_option(section, "DEPENDENCIES"):
                continue
            dependencies = parser.get(section, "DEPENDENCIES").split()
            dep_section = dict()
            dep_distance = dict()
            dep_running = dict()
            for dependency in dependencies:
                if '-' in dependency:
                    dependency_split = dependency.split('-')
                    dep_section[dependency] = dependency_split[0]
                    dep_distance[dependency] = int(dependency_split[1])
                    dep_running[dependency] = dic_jobs.get_option(dependency_split[0], 'RUNNING', 'once').lower()
                else:
                    dep_section[dependency] = dependency

            for job in dic_jobs.get_jobs(section):
                for dependency in dependencies:
                    chunk = job.chunk
                    member = job.member
                    date = job.date

                    section_name = dep_section[dependency]

                    if '-' in dependency:
                        distance = dep_distance[dependency]
                        if chunk is not None and dep_running[dependency] == 'chunk':
                            chunk_index = chunk_list.index(chunk)
                            if chunk_index >= distance:
                                chunk = chunk_list[chunk_index - distance]
                            else:
                                continue
                        elif member is not None and dep_running[dependency] in ['chunk', 'member']:
                            member_index = member_list.index(member)
                            if member_index >= distance:
                                member = member_list[member_index - distance]
                            else:
                                continue
                        elif date is not None and dep_running[dependency] in ['chunk', 'member', 'startdate']:
                            date_index = date_list.index(date)
                            if date_index >= distance:
                                date = date_list[date_index - distance]
                            else:
                                continue

                    for parent in dic_jobs.get_jobs(section_name, date, member, chunk):
                        job.add_parent(parent)

                    if job.wait and job.frequency > 1:
                        if job.chunk is not None:
                            max_distance = (chunk_list.index(chunk)+1) % job.frequency
                            if max_distance == 0:
                                max_distance = job.frequency
                            for distance in range(1, max_distance, 1):
                                for parent in dic_jobs.get_jobs(section_name, date, member, chunk - distance):
                                    job.add_parent(parent)
                        elif job.member is not None:
                            max_distance = (member_list.index(chunk)+1) % job.frequency
                            if max_distance == 0:
                                max_distance = job.frequency
                            for distance in range(1, max_distance, 1):
                                for parent in dic_jobs.get_jobs(section_name, date, member - distance, chunk):
                                    job.add_parent(parent)
                        elif job.date is not None:
                            max_distance = (date_list.index(chunk)+1) % job.frequency
                            if max_distance == 0:
                                max_distance = job.frequency
                            for distance in range(1, max_distance, 1):
                                for parent in dic_jobs.get_jobs(section_name, date - distance, member, chunk):
                                    job.add_parent(parent)

        Log.info("Removing redundant dependencies...")
        self.update_genealogy()
        for job in self._job_list:
            job.parameters = parameters

    def __len__(self):
        return self._job_list.__len__()

    def get_job_list(self):
        return self._job_list

    def get_completed(self):
        """Returns a list of completed jobs"""
        return [job for job in self._job_list if job.status == Status.COMPLETED]

    def get_submitted(self):
        """Returns a list of submitted jobs"""
        return [job for job in self._job_list if job.status == Status.SUBMITTED]

    def get_running(self):
        """Returns a list of jobs running"""
        return [job for job in self._job_list if job.status == Status.RUNNING]

    def get_queuing(self):
        """Returns a list of jobs queuing"""
        return [job for job in self._job_list if job.status == Status.QUEUING]

    def get_failed(self):
        """Returns a list of failed jobs"""
        return [job for job in self._job_list if job.status == Status.FAILED]

    def get_ready(self):
        """Returns a list of jobs ready"""
        return [job for job in self._job_list if job.status == Status.READY]

    def get_waiting(self):
        """Returns a list of jobs waiting"""
        return [job for job in self._job_list if job.status == Status.WAITING]

    def get_unknown(self):
        """Returns a list of jobs unknown"""
        return [job for job in self._job_list if job.status == Status.UNKNOWN]

    def get_in_queue(self):
        """Returns a list of jobs in the queue (Submitted, Running, Queuing)"""
        return self.get_submitted() + self.get_running() + self.get_queuing()

    def get_not_in_queue(self):
        """Returns a list of jobs NOT in the queue (Ready, Waiting)"""
        return self.get_ready() + self.get_waiting()

    def get_finished(self):
        """Returns a list of jobs finished (Completed, Failed)"""
        return self.get_completed() + self.get_failed()

    def get_active(self):
        """Returns a list of active jobs (In queue, Ready)"""
        return self.get_in_queue() + self.get_ready() + self.get_unknown()

    def get_job_by_name(self, name):
        """Returns the job that its name matches name"""
        for job in self._job_list:
            if job.name == name:
                return job
        Log.warning("We could not find that job %s in the list!!!!", name)

    def sort_by_name(self):
        return sorted(self._job_list, key=lambda k: k.name)

    def sort_by_id(self):
        return sorted(self._job_list, key=lambda k: k.id)

    def sort_by_type(self):
        return sorted(self._job_list, key=lambda k: k.type)

    def sort_by_status(self):
        return sorted(self._job_list, key=lambda k: k.status)

    @staticmethod
    def load_file(filename):
        if os.path.exists(filename):
            return pickle.load(file(filename, 'r'))
        else:
            # URi: print ERROR
            return list()

    def load(self):
        Log.info("Loading JobList: " + self._pkl_path + self._job_list_file)
        return JobList.load_file(self._pkl_path + self._job_list_file)

    def load_updated(self):
        Log.info("Loading updated list: " + self._pkl_path + self._update_file)
        return JobList.load_file(self._pkl_path + self._update_file)

    def load_failed(self):
        Log.info("Loading failed list: " + self._pkl_path + self._failed_file)
        return JobList.load_file(self._pkl_path + self._failed_file)

    def save_failed(self, failed_list):
        # URi: should we check that the path exists?
        Log.info("Saving failed list: " + self._pkl_path + self._failed_file)
        pickle.dump(failed_list, file(self._pkl_path + self._failed_file, 'w'))

    def save(self):
        # URi: should we check that the path exists?
        setrecursionlimit(50000)
        Log.debug("Saving JobList: " + self._pkl_path + self._job_list_file)
        pickle.dump(self, file(self._pkl_path + self._job_list_file, 'w'))

    def update_from_file(self, store_change=True):
        if os.path.exists(self._pkl_path + self._update_file):
            for line in open(self._pkl_path + self._update_file):
                if line.strip() == '':
                    continue
                job = self.get_job_by_name(line.split()[0])
                if job:
                    job.status = self._stat_val.retval(line.split()[1])
                    job.fail_count = 0
            now = localtime()
            output_date = strftime("%Y%m%d_%H%M", now)
            if store_change:
                move(self._pkl_path + self._update_file, self._pkl_path + self._update_file + "_" + output_date)

    def update_parameters(self, parameters):
        self._parameters = parameters
        for job in self._job_list:
            job.parameters = parameters

    def update_list(self, store_change=True):
        # load updated file list
        self.update_from_file(store_change)

        # reset jobs that has failed less ethan 10 times
        if 'RETRIALS' in self._parameters:
            retrials = int(self._parameters['RETRIALS'])
        else:
            retrials = 4

        for job in self.get_failed():
            job.inc_fail_count()
            if job.fail_count < retrials:
                job.status = Status.READY

        # if waiting jobs has all parents completed change its State to READY
        for job in self.get_waiting():
            tmp = [parent for parent in job.parents if parent.status == Status.COMPLETED]
            # for parent in job.parents:
            # if parent.status != Status.COMPLETED:
            # break
            if len(tmp) == len(job.parents):
                job.status = Status.READY
        if store_change:
            self.save()

    def update_shortened_names(self):
        """In some cases the scheduler only can operate with names shorter than 15 characters.
        Update the job list replacing job names by the corresponding shortened job name"""
        for job in self._job_list:
            job.name = job.short_name

    def update_genealogy(self):
        """When we have created the joblist, every type of job is created.
        Update genealogy remove jobs that have no templates"""

        # Use a copy of job_list because original is modified along iterations
        for job in self._job_list[:]:
            if job.file is None or job.file == '':
                self._remove_job(job)

        # Simplifing dependencies: if a parent is alreaday an ancestor of another parent,
        # we remove parent dependency
        for job in self._job_list:
            for parent in list(job.parents):
                for ancestor in parent.ancestors:
                    if ancestor in job.parents:
                        job.parents.remove(ancestor)
                        ancestor.children.remove(job)

        for job in self._job_list:
            if not job.has_parents():
                job.status = Status.READY

    def check_scripts(self, as_conf):
        """When we have created the scripts, all parameters should have been substituted.
        %PARAMETER% handlers not allowed"""
        out = True
        for job in self._job_list:
            if not job.check_script(as_conf):
                out = False
                Log.warning("Invalid parameter substitution in %s!!!" % job.name)

        return out

    def _remove_job(self, job):
        for child in job.children:
            for parent in job.parents:
                child.add_parent(parent)
            child.delete_parent(job)

        for parent in job.parents:
            parent.children.remove(job)

        self._job_list.remove(job)

    def rerun(self, chunk_list):
        parser = SafeConfigParser()
        parser.optionxform = str
        parser.read(os.path.join(BasicConfig.LOCAL_ROOT_DIR, self._expid, 'conf', "jobs_" + self._expid + ".conf"))

        Log.info("Adding dependencies...")
        dep_section = dict()
        dep_distance = dict()
        dependencies = dict()
        dep_running = dict()
        for section in parser.sections():
            Log.debug("Reading rerun dependencies for {0} jobs".format(section))
            if not parser.has_option(section, "RERUN_DEPENDENCIES"):
                continue
            dependencies[section] = parser.get(section, "RERUN_DEPENDENCIES").split()
            dep_section[section] = dict()
            dep_distance[section] = dict()
            dep_running[section] = dict()
            for dependency in dependencies[section]:
                if '-' in dependency:
                    dependency_split = dependency.split('-')
                    dep_section[section][dependency] = dependency_split[0]
                    dep_distance[section][dependency] = int(dependency_split[1])
                    dep_running[section][dependency] = self._dic_jobs.get_option(dependency_split[0], 'RUNNING',
                                                                                 'once').lower()
                else:
                    dep_section[section][dependency] = dependency

        for job in self._job_list:
            job.status = Status.COMPLETED

        data = json.loads(chunk_list)
        for d in data['sds']:
            date = d['sd']
            Log.debug("Date: " + date)
            for m in d['ms']:
                member = m['m']
                Log.debug("Member: " + member)
                for c in m['cs']:
                    Log.debug("Chunk: " + c)
                    chunk = int(c)
                    for job in [i for i in self._job_list if i.date == date and i.member == member
                                and i.chunk == chunk]:
                        job.status = Status.WAITING
                        Log.debug("Job: " + job.name)
                        section = job.section
                        if section not in dependencies:
                            continue
                        for dependency in dependencies[section]:
                            current_chunk = chunk
                            current_member = member
                            current_date = date
                            if '-' in dependency:
                                distance = dep_distance[section][dependency]
                                running = dep_running[section][dependency]
                                if current_chunk is not None and running == 'chunk':
                                    chunk_index = self._chunk_list.index(current_chunk)
                                    if chunk_index >= distance:
                                        current_chunk = self._chunk_list[chunk_index - distance]
                                    else:
                                        continue
                                elif current_member is not None and running in ['chunk', 'member']:
                                    member_index = self._member_list.index(current_member)
                                    if member_index >= distance:
                                        current_member = self._member_list[member_index - distance]
                                    else:
                                        continue
                                elif current_date is not None and running in ['chunk', 'member', 'startdate']:
                                    date_index = self._date_list.index(current_date)
                                    if date_index >= distance:
                                        current_date = self._date_list[date_index - distance]
                                    else:
                                        continue
                            section_name = dep_section[section][dependency]
                            for parent in self._dic_jobs.get_jobs(section_name, current_date, current_member,
                                                                  current_chunk):
                                parent.status = Status.WAITING
                                Log.debug("Parent: " + parent.name)

        for job in [j for j in self._job_list if j.status == Status.COMPLETED]:
            self._remove_job(job)

        self.update_genealogy()


class DicJobs:

    def __init__(self, joblist, parser, date_list, member_list, chunk_list):
        self._date_list = date_list
        self._joblist = joblist
        self._member_list = member_list
        self._chunk_list = chunk_list
        self._parser = parser
        self._dic = dict()

    def read_section(self, section, priority):
        running = 'once'
        if self._parser.has_option(section, 'RUNNING'):
            running = self._parser.get(section, 'RUNNING').lower()
        frequency = int(self.get_option(section, "FREQUENCY", 1))
        if running == 'once':
            self._create_jobs_once(section, priority)
        elif running == 'startdate':
            self._create_jobs_startdate(section, priority, frequency)
        elif running == 'member':
            self._create_jobs_member(section, priority, frequency)
        elif running == 'chunk':
            self._create_jobs_chunk(section, priority, frequency)

    def _create_jobs_once(self, section, priority):
        self._dic[section] = self._create_job(section, priority, None, None, None)

    def _create_jobs_startdate(self, section, priority, frequency):
        self._dic[section] = dict()
        count = 0
        for date in self._date_list:
            count += 1
            if count % frequency == 0 or count == len(self._date_list):
                self._dic[section][date] = self._create_job(section, priority, date, None, None)

    def _create_jobs_member(self, section, priority, frequency):
        self._dic[section] = dict()
        for date in self._date_list:
            self._dic[section][date] = dict()
            count = 0
            for member in self._member_list:
                count += 1
                if count % frequency == 0 or count == len(self._member_list):
                    self._dic[section][date][member] = self._create_job(section, priority, date, member, None)

    def _create_jobs_chunk(self, section, priority, frequency):
        self._dic[section] = dict()
        for date in self._date_list:
            self._dic[section][date] = dict()
            for member in self._member_list:
                self._dic[section][date][member] = dict()
                count = 0
                for chunk in self._chunk_list:
                    count += 1
                    if count % frequency == 0 or count == len(self._chunk_list):
                        self._dic[section][date][member][chunk] = self._create_job(section, priority, date, member,
                                                                                   chunk)

    def get_jobs(self, section, date=None, member=None, chunk=None):
        jobs = list()
        dic = self._dic[section]
        if type(dic) is not dict:
            jobs.append(dic)
        else:
            if date is not None:
                self._get_date(jobs, dic, date, member, chunk)
            else:
                for d in self._date_list:
                    self._get_date(jobs, dic, d, member, chunk)
        return jobs

    def _get_date(self, jobs, dic, date, member, chunk):
        if date not in dic:
            return jobs
        dic = dic[date]
        if type(dic) is not dict:
            jobs.append(dic)
        else:
            if member is not None:
                self._get_member(jobs, dic, member, chunk)
            else:
                for m in self._member_list:
                    self._get_member(jobs, dic, m, chunk)

        return jobs

    def _get_member(self, jobs, dic, member, chunk):
        if member not in dic:
            return jobs
        dic = dic[member]
        if type(dic) is not dict:
            jobs.append(dic)
        else:
            if chunk is not None and chunk in dic:
                jobs.append(dic[chunk])
            else:
                for c in self._chunk_list:
                    if c not in dic:
                        continue
                    jobs.append(dic[c])
        return jobs

    def _create_job(self, section, priority, date, member, chunk):
        name = self._joblist.expid
        if date is not None:
            name += "_" + date
        if member is not None:
            name += "_" + member
        if chunk is not None:
            name += "_{0}".format(chunk)
        name += "_" + section
        job = Job(name, 0, Status.WAITING, priority)
        job.section = section
        job.date = date
        job.member = member
        job.chunk = chunk

        job.frequency = int(self.get_option(section, "FREQUENCY", 1))
        job.wait = bool(self.get_option(section, "WAIT", False))

        job.queue_name = self.get_option(section, "QUEUE", None)
        if job.queue_name is not None:
            job.queue_name = job.queue_name.lower()
        job.file = self.get_option(section, "FILE", None)

        job.processors = self.get_option(section, "PROCESSORS", 1)
        job.threads = self.get_option(section, "THREADS", 1)
        job.tasks = self.get_option(section, "TASKS", 1)

        job.wallclock = self.get_option(section, "WALLCLOCK", '')
        self._joblist.get_job_list().append(job)
        return job

    def get_option(self, section, option, default):
        if self._parser.has_option(section, option):
            return self._parser.get(section, option)
        else:
            return default




#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.

from os import path
from setuptools import setup
from setuptools import find_packages

here = path.abspath(path.dirname(__file__))

# Get the version number from the relevant file
with open(path.join(here, 'VERSION')) as f:
    version = f.read().strip()

setup(
    name='autosubmit',
    license='GNU GPL v3',
    platforms=['GNU/Linux Debian'],
    version=version,
    description='Autosubmit: a versatile tool for managing Global Climate Coupled Models in '
                'Supercomputing Environments',
    author='Domingo Manubens-Gil',
    author_email='domingo.manubens@ic3.cat',
    url='https://autosubmit.ic3.cat',
    download_url='http://ic3.cat/wikicfu/index.php/Tools/Autosubmit',
    keywords=['climate', 'workflow', 'HPC'],
    install_requires=['argparse>=1.2', 'python-dateutil>=1,<2', 'pydotplus', 'pyparsing'],
    # 'numpy','matplotlib>=1.1.1',
    packages=find_packages(),
    include_package_data=True,
    package_data={'autosubmit': [
        'autosubmit/config/files/autosubmit.conf',
        'autosubmit/config/files/expdef.conf',
        'autosubmit/database/data/autosubmit.sql'
    ]
    },
	scripts=['bin/autosubmit.py'],
)



#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import textwrap

from autosubmit.queue.hpcqueue import HPCQueue


class LsfQueue(HPCQueue):
    def __init__(self, expid):
        HPCQueue.__init__(self)
        self._host = "mn-ecm86"
        self.scratch = ""
        self.project = ""
        self.user = ""
        self._header = LsfHeader()
        self.expid = expid
        self.job_status = dict()
        self.job_status['COMPLETED'] = ['DONE']
        self.job_status['RUNNING'] = ['RUN']
        self.job_status['QUEUING'] = ['PEND', 'FW_PEND']
        self.job_status['FAILED'] = ['SSUSP', 'USUSP', 'EXIT']
        self.update_cmds()

    def update_cmds(self):
        self.remote_log_dir = (self.scratch + "/" + self.project + "/" + self.user + "/" + self.expid + "/LOG_"
                               + self.expid)
        self.cancel_cmd = "ssh " + self._host + " bkill"
        self._checkjob_cmd = "ssh " + self._host + " bjobs "
        self._checkhost_cmd = "ssh " + self._host + " echo 1"
        self._submit_cmd = "ssh " + self._host + " bsub \< " + self.remote_log_dir + "/"
        self.put_cmd = "scp"
        self.get_cmd = "scp"
        self.mkdir_cmd = "ssh " + self._host + " mkdir -p " + self.remote_log_dir

    def get_checkhost_cmd(self):
        return self._checkhost_cmd

    def get_mkdir_cmd(self):
        return self.mkdir_cmd

    def get_remote_log_dir(self):
        return self.remote_log_dir

    def parse_job_output(self, output):
        job_state = output.split('\n')[1].split()[2]
        return job_state

    def get_submitted_job_id(self, output):
        return output.split('<')[1].split('>')[0]

    def jobs_in_queue(self, output):
        return zip(*[line.split() for line in output.split('\n')])[0][1:]

    def get_checkjob_cmd(self, job_id):
        return self._checkjob_cmd + str(job_id)

    def get_submit_cmd(self, job_script):
        return self._submit_cmd + job_script


class LsfHeader:
    """Class to handle the BSC headers of a job"""

    SERIAL = textwrap.dedent("""
            #!/bin/ksh
            ###############################################################################
            #                     %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #@ job_name         = %JOBNAME%
            #@ wall_clock_limit = %WALLCLOCK%
            #@ output           = %SCRATCH_DIR%/%HPCUSER%/%EXPID%/LOG_%EXPID%/%JOBNAME%_%j.out
            #@ error            = %SCRATCH_DIR%/%HPCUSER%/%EXPID%/LOG_%EXPID%/%JOBNAME%_%j.err
            #@ total_tasks      = %NUMTASK%
            #@ initialdir       = %SCRATCH_DIR%/%HPCUSER%/%EXPID%/
            #@ class            = %CLASS%
            #@ partition        = %PARTITION%
            #@ features         = %FEATURES%
            #
            ###############################################################################
            """)

    PARALLEL = textwrap.dedent("""
            #!/bin/ksh
            ###############################################################################
            #                     %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #@ job_name         = %JOBNAME%
            #@ wall_clock_limit = %WALLCLOCK%
            #@ output           = %SCRATCH_DIR%/%HPCUSER%/%EXPID%/LOG_%EXPID%/%JOBNAME%_%j.out
            #@ error            = %SCRATCH_DIR%/%HPCUSER%/%EXPID%/LOG_%EXPID%/%JOBNAME%_%j.err
            #@ total_tasks      = %NUMTASK%
            #@ initialdir       = %SCRATCH_DIR%/%HPCUSER%/%EXPID%/
            #@ tasks_per_node   = %TASKSNODE%
            #@ tracing          = %TRACING%
            #
            ###############################################################################
            """)


#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.


from commands import getstatusoutput
import os
from time import sleep
from sys import exit

from autosubmit.job.job_common import Status
from autosubmit.config.basicConfig import BasicConfig
from autosubmit.config.log import Log


SLEEPING_TIME = 30


class HPCQueue:
    def __init__(self):
        self._submit_cmd = None
        self.get_cmd = None
        self._checkjob_cmd = None
        self.job_status = None
        self.expid = None
        self.put_cmd = None
        self.mkdir_cmd = None
        self.cancel_cmd = None
        self._header = None
        self._serial_queue = None

    @property
    def header(self):
        return self._header

    def get_serial_queue(self):
        if self._serial_queue is None:
            return self
        else:
            return self._serial_queue

    def set_serial_queue(self, value):
        self._serial_queue = value

    def cancel_job(self, job_id):
        Log.debug(self.cancel_cmd + ' ' + str(job_id))
        # noinspection PyUnusedLocal
        (status, output) = getstatusoutput(self.cancel_cmd + ' ' + str(job_id))

    def check_job(self, job_id):
        job_status = Status.UNKNOWN

        if type(job_id) is not int:
            # URi: logger
            Log.error('check_job() The job id (%s) is not an integer.' % job_id)
            # URi: value ?
            return job_status

        retry = 10
        (status, output) = getstatusoutput(self.get_checkjob_cmd(job_id))
        Log.debug(status)
        Log.debug(output)
        # retry infinitelly except if it was in the RUNNING state, because it can happen that we don't get a
        # COMPLETE status from autosubmit.queue due to the 5 min lifetime
        while status != 0 and retry > 0:
            # if(current_state == Status.RUNNING):
            retry -= 1
            Log.info('Can not get job status, retrying in 10 sec')
            (status, output) = getstatusoutput(self.get_checkjob_cmd(job_id))
            Log.debug(status)
            Log.debug(output)
            # URi: logger
            sleep(10)

        if status == 0:
            # URi: this command is specific of mn
            job_status = self.parse_job_output(output)
            # URi: define status list in HPC Queue Class
            if job_status in self.job_status['COMPLETED'] or retry == 0:
                job_status = Status.COMPLETED
            elif job_status in self.job_status['RUNNING']:
                job_status = Status.RUNNING
            elif job_status in self.job_status['QUEUING']:
                job_status = Status.QUEUING
            elif job_status in self.job_status['FAILED']:
                job_status = Status.FAILED
            else:
                job_status = Status.UNKNOWN
        else:
            # BOUOUOUOU	NOT	GOOD!
            job_status = Status.UNKNOWN
        return job_status

    def check_host(self):
        (status, output) = getstatusoutput(self.get_checkhost_cmd())
        if status == 0:
            Log.debug('The host ' + self._host + ' is up')
            return True
        else:
            Log.debug('The host ' + self._host + ' is down')
            return False

    def check_remote_log_dir(self):
        (status, output) = getstatusoutput(self.get_mkdir_cmd())
        Log.debug(self.mkdir_cmd)
        Log.debug(output)
        if status == 0:
            Log.debug('%s has been created on %s .' % (self.remote_log_dir, self._host))
        else:
            Log.error('Could not create the DIR on HPC')

    def send_script(self, job_script):
        (status, output) = getstatusoutput(self.put_cmd + ' ' + BasicConfig.LOCAL_ROOT_DIR + "/" + self.expid +
                                           '/tmp/' + str(job_script) + ' ' + self._host + ':' + self.remote_log_dir +
                                           "/" + str(job_script))
        Log.debug(self.put_cmd + ' ' + BasicConfig.LOCAL_ROOT_DIR + "/" + self.expid + '/tmp/' + str(
            job_script) + ' ' + self._host + ':' + self.remote_log_dir + "/" + str(job_script))
        if status == 0:
            Log.debug('The script has been sent')
        else:
            Log.error('The script has not been sent')

    def get_completed_files(self, jobname):

        filename = jobname + '_COMPLETED'
        completed_local_path = os.path.join(BasicConfig.LOCAL_ROOT_DIR, self.expid, BasicConfig.LOCAL_TMP_DIR, filename)
        if os.path.exists(completed_local_path):
            os.remove(completed_local_path)
        completed_remote_path = os.path.join(self.remote_log_dir, filename)
        (status, output) = getstatusoutput(self.get_cmd + ' ' + self._host + ':' + completed_remote_path +
                                           ' ' + completed_local_path)
        Log.debug(self.get_cmd + ' ' + self._host + ':' + self.remote_log_dir + '/' + filename + ' ' +
                  completed_local_path)
        if status == 0:
            Log.debug('The COMPLETED files have been transfered')
            return True

        # wait five seconds to check get file
        sleep(5)
        (status, output) = getstatusoutput(self.get_cmd + ' ' + self._host + ':' + completed_remote_path +
                                           ' ' + completed_local_path)
        if status == 0:
            Log.debug('The COMPLETED files have been transfered')
            return True
        Log.debug('Something did not work well when transferring the COMPLETED files')
        return False

    def submit_job(self, job_script):
        (status, output) = getstatusoutput(self.get_submit_cmd(job_script))
        if status == 0:
            job_id = self.get_submitted_job_id(output)
            Log.debug(job_id)
            return int(job_id)

    def normal_stop(self):
        sleep(SLEEPING_TIME)
        (status, output) = getstatusoutput(self.get_checkjob_cmd(' '))
        for job_id in self.jobs_in_queue(output):
            self.cancel_job(job_id)

        exit(0)

    def smart_stop(self):
        sleep(SLEEPING_TIME)
        (status, output) = getstatusoutput(self.get_checkjob_cmd(' '))
        Log.debug(self.jobs_in_queue(output))
        while self.jobs_in_queue(output):
            Log.debug(self.jobs_in_queue(output))
            sleep(SLEEPING_TIME)
            (status, output) = getstatusoutput(self.get_checkjob_cmd(' '))
        exit(0)

    def set_host(self, new_host):
        self._host = new_host

    def set_scratch(self, new_scratch):
        self.scratch = new_scratch

    def set_project(self, new_project):
        self.project = new_project

    def set_user(self, new_user):
        self.user = new_user

    def set_remote_log_dir(self, new_remote_log_dir):
        self.remote_log_dir = new_remote_log_dir

    def get_checkhost_cmd(self):
        raise NotImplementedError

    def get_mkdir_cmd(self):
        raise NotImplementedError

    def parse_job_output(self, output):
        raise NotImplementedError

    def jobs_in_queue(self, output):
        raise NotImplementedError

    def get_submitted_job_id(self, output):
        raise NotImplementedError

    def get_header(self, job):
        if job.processors > 1:
            return self.header.PARALLEL
        else:
            return self.header.SERIAL

    def get_checkjob_cmd(self, job_id):
        raise NotImplementedError

    def get_submit_cmd(self, job_script):
        raise NotImplementedError

    def get_shcall(self, job_script):
        return '"nohup /bin/sh {0} > {0}.stdout 2> {0}.stderr & echo \$!"'.format(os.path.join(self.remote_log_dir,
                                                                                               job_script))

    @staticmethod
    def get_pscall(job_id):
        return '"kill -0 {0} > {0}.stat.stdout 2> {0}.stat.stderr; echo \$?"'.format(job_id)

    @staticmethod
    def get_qstatjob(job_id):
        return '''"if [[ \$(qstat | grep {0}) != '' ]];
        then echo \$(qstat | grep {0} | awk '{{print \$5}}' | head -n 1); else echo 'c'; fi"'''.format(job_id)


#!/usr/bin/env python

# Copyright 2014 Climate Forecasting Unit, IC3

# This file is part of Autosubmit.

# Autosubmit is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Autosubmit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Autosubmit.  If not, see <http://www.gnu.org/licenses/>.
import textwrap

from autosubmit.queue.hpcqueue import HPCQueue
from autosubmit.config.log import Log


class PBSQueue(HPCQueue):

    def __init__(self, expid, version):
        HPCQueue.__init__(self)
        self._host = ""
        self._version = version
        self.scratch = ""
        self.project = ""
        self.user = ""

        if str.startswith(version, '10'):
            self._header = Pbs10Header()
        elif str.startswith(version, '11'):
            self._header = Pbs11Header()
        elif str.startswith(version, '12'):
            self._header = Pbs12Header()
        else:
            Log.error('PBS version {0} not supported'.format(version))
            exit(1)

        self.expid = expid
        self.job_status = dict()
        self.job_status['COMPLETED'] = ['F', 'E', 'c', 'C']
        self.job_status['RUNNING'] = ['R']
        self.job_status['QUEUING'] = ['Q', 'H', 'S', 'T', 'W', 'U', 'M']
        self.job_status['FAILED'] = ['Failed', 'Node_fail', 'Timeout']
        self.update_cmds()

    def update_cmds(self):
        self.remote_log_dir = (self.scratch + "/" + self.project + "/" + self.user + "/" +
                               self.expid + "/LOG_" + self.expid)
        self.cancel_cmd = "ssh " + self._host + " qdel"
        self._checkhost_cmd = "ssh " + self._host + " echo 1"
        self.put_cmd = "scp"
        self.get_cmd = "scp"
        self.mkdir_cmd = "ssh " + self._host + " mkdir -p " + self.remote_log_dir
        self._submit_cmd = "ssh " + self._host + " qsub -d " + self.remote_log_dir + " " + self.remote_log_dir + "/ "

        if str.startswith(self._version, '11'):
            self._checkjob_cmd = "ssh " + self._host + " qstat"

    def get_checkhost_cmd(self):
        return self._checkhost_cmd

    def get_remote_log_dir(self):
        return self.remote_log_dir

    def get_mkdir_cmd(self):
        return self.mkdir_cmd

    def parse_job_output(self, output):
        # job_state = output.split('\n')[2].split()[4]
        # return job_state
        return output

    def get_submitted_job_id(self, output):
        return output.split('.')[0]

    def jobs_in_queue(self, output):
        Log.debug(output)
        return output.split()

    def get_submit_cmd(self, job_script):
        return self._submit_cmd + job_script

    def get_checkjob_cmd(self, job_id):
        if str.startswith(self._version, '11'):
            return self._checkjob_cmd + str(job_id)
        else:
            return "ssh " + self._host + " " + self.get_qstatjob(job_id)


class Pbs12Header:
    """Class to handle the Archer headers of a job"""

    SERIAL = textwrap.dedent("""
            #!/bin/sh
            ###############################################################################
            #                   %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #PBS -N %JOBNAME%
            #PBS -l select=serial=true:ncpus=1
            #PBS -l walltime=%WALLCLOCK%:00
            #PBS -A %HPCPROJ%
            #
            ###############################################################################
            """)

    PARALLEL = textwrap.dedent("""
            #!/bin/sh
            ###############################################################################
            #                   %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #PBS -N %JOBNAME%
            #PBS -l select=%NUMPROC%
            #PBS -l walltime=%WALLCLOCK%:00
            #PBS -A %HPCPROJ%
            #
            ###############################################################################
            """)


class Pbs10Header:
    """Class to handle the Hector headers of a job"""

    SERIAL = textwrap.dedent("""
            #!/bin/sh
            ###############################################################################
            #                   %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #PBS -N %JOBNAME%
            #PBS -q serial
            #PBS -l cput=%WALLCLOCK%:00
            #PBS -A %HPCPROJ%
            #
            ###############################################################################
            """)

    PARALLEL = textwrap.dedent("""
            #!/bin/sh
            ###############################################################################
            #                   %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #PBS -N %JOBNAME%
            #PBS -l mppwidth=%NUMPROC%
            #PBS -l mppnppn=32
            #PBS -l walltime=%WALLCLOCK%:00
            #PBS -A %HPCPROJ%
            #
            ###############################################################################
            """)


class Pbs11Header:
    """Class to handle the Lindgren headers of a job"""

    SERIAL = textwrap.dedent("""\
            #!/bin/sh
            ###############################################################################
            #                         %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #!/bin/sh --login
            #PBS -N %JOBNAME%
            #PBS -l mppwidth=%NUMPROC%
            #PBS -l mppnppn=%NUMTASK%
            #PBS -l walltime=%WALLCLOCK%
            #PBS -e %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%
            #PBS -o %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%
            #
            ###############################################################################
            """)

    PARALLEL = textwrap.dedent("""\
            #!/bin/sh
            ###############################################################################
            #                         %TASKTYPE% %EXPID% EXPERIMENT
            ###############################################################################
            #
            #!/bin/sh --login
            #PBS -N %JOBNAME%
            #PBS -l mppwidth=%NUMPROC%
            #PBS -l mppnppn=%NUMTASK%
            #PBS -l walltime=%WALLCLOCK%
            #PBS -e %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%
            #PBS -o %SCRATCH_DIR%/%HPCPROJ%/%HPCUSER%/%EXPID%/LOG_%EXPID%
            #
            ###############################################################################
            """)